package com.yfd.platform.modules.auxcontrol.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.ObjUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yfd.platform.modules.auxcontrol.domain.DeviceAlarmParameter;
import com.yfd.platform.modules.auxcontrol.domain.DeviceAlarmRecord;
import com.yfd.platform.modules.auxcontrol.domain.MeterDevice;
import com.yfd.platform.modules.auxcontrol.mapper.DeviceAlarmParameterMapper;
import com.yfd.platform.modules.auxcontrol.mapper.DeviceAlarmRecordMapper;
import com.yfd.platform.modules.auxcontrol.mapper.DeviceSignalMapper;
import com.yfd.platform.modules.auxcontrol.mapper.MeterDeviceMapper;
import com.yfd.platform.modules.auxcontrol.service.IDeviceAlarmRecordService;
import com.yfd.platform.system.domain.SysDictionaryItems;
import com.yfd.platform.system.mapper.SysDictionaryItemsMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * <p>
 * 变电站-辅控设备-告警记录 服务实现类
 * </p>
 *
 * @author zhengsl
 * @since 2025-04-23
 */
@Service
public class DeviceAlarmRecordServiceImpl extends ServiceImpl<DeviceAlarmRecordMapper, DeviceAlarmRecord> implements IDeviceAlarmRecordService {

    @Resource
    private DeviceSignalMapper deviceSignalMapper;

    @Resource
    private MeterDeviceMapper meterDeviceMapper;

    @Resource
    private DeviceAlarmParameterMapper deviceAlarmParameterMapper;

    @Resource
    private DeviceAlarmRecordMapper deviceAlarmRecordMapper;

    @Resource
    private SysDictionaryItemsMapper sysDictionaryItemsMapper;

    /**********************************
     * 用途说明: 生成报警记录（IEC104）
     * 参数说明
     * from  'IEC104','IEC61850'
     * type  'yx','yc'
     * slave_ip     IP
     * address     遥信、要测地址
     * value         值
     * 返回值说明: 无返回值
     ***********************************/
    @Override
    @Transactional
    public void doAlaramRecord(String from, String type, String slaveIp, String address, String value) {
        String sinnalType = type;
        if ("IEC104".equals(from)) {
            sinnalType = "yc";
        } else if ("IEC61850".equals(from)) {
            slaveIp = null;
        }
        Map<String, Object> map = deviceSignalMapper.selectDeviceSignalMap(slaveIp, sinnalType, address);
        if (map == null || ObjUtil.isEmpty(map)) {
            return;
        }
        //如果是yx 遥信  （设备自身报警记录）
        if ("yx".equals(type)) {
            //如果信号类型是1  设备告警
            if ("1".equals(map.get("signalType").toString())) {
                //紧接者判断  value （1是告警 0是正常） status
                if (1 == Integer.parseInt(value)) {
                    //填充告警记录信息接口
                    DeviceAlarmRecord deviceAlarmRecord = FillingDeviceAlarmRecord(type, null, map, value);
                    //首先判断 在不在时间范围之内  判断时间在不在时间范围之内接口
                    Map<String, Object> timeInterval = getTimeInterval(7);
                    //查询表中是否存在该报警记录
                    List<DeviceAlarmRecord> alarmRecords = queryDeviceAlarmRecord(deviceAlarmRecord, timeInterval);
                    //如果没有有报警记录 新增记录
                    if (alarmRecords.size() == 0) {
                        doAlarmAction(deviceAlarmRecord);
                    } else if (alarmRecords.size() > 1) {
                        this.removeById(alarmRecords.get(0).getRecordId());
                    }
                } else {
                    //紧接者判断  value （1是告警 0是正常） status  同一信号监测到报警状态消除，对当前信号所有未处理历史报警打上已处理标签，并记录消除时间
                    LambdaQueryWrapper<DeviceAlarmRecord> queryWrapper = new LambdaQueryWrapper<>();
                    queryWrapper.eq(DeviceAlarmRecord::getSignalId, map.get("signalId").toString());
                    queryWrapper.eq(DeviceAlarmRecord::getAlarmClass, "2");
                    queryWrapper.in(DeviceAlarmRecord::getStatus, "01", "02");
                    DeviceAlarmRecord deviceAlarmRecord = new DeviceAlarmRecord();
                    deviceAlarmRecord.setStatus("09");
                    //状态 故障消除
                    deviceAlarmRecord.setFixTime(DateUtil.toLocalDateTime(DateUtil.date()));
                    //故障消除时间
                    this.update(deviceAlarmRecord, queryWrapper);
                }
            }
        } else if ("yc".equals(type)) {
            //1.通过设备id 和信号id 查询告警参数设置表（告警规则）
            LambdaQueryWrapper<DeviceAlarmParameter> querywrapperdap = new LambdaQueryWrapper<>();
            if (StrUtil.isNotEmpty(map.get("meterDeviceId").toString())) {
                querywrapperdap.eq(DeviceAlarmParameter::getDeviceId, map.get("meterDeviceId").toString());
            }
            if (StrUtil.isNotEmpty(map.get("signalId").toString())) {
                querywrapperdap.eq(DeviceAlarmParameter::getSignalId, map.get("signalId").toString());
            }
            List<DeviceAlarmParameter> deviceAlarmParameters = deviceAlarmParameterMapper.selectList(querywrapperdap);
            if (deviceAlarmParameters != null && deviceAlarmParameters.size() > 0) {
                for (DeviceAlarmParameter deviceAlarmParameter : deviceAlarmParameters) {
                    //如果告警触发条件 不为空  1<=value<=100
                    if (deviceAlarmParameter.getAlarmCondition() != null) {
                        //根据告警规则 判断是否触发告警
                        boolean alarmTrigger = TriggerAlarm(deviceAlarmParameter, value);

                        //2.通过告警触发条件 和 value比较 满足条件 触发告警
                        if (alarmTrigger) {
                            DeviceAlarmRecord deviceAlarmRecord = FillingDeviceAlarmRecord(type, deviceAlarmParameter
                                    , map, value);
                            //首先判断 在不在时间范围之内  判断时间在不在时间范围之内接口
                            Map<String, Object> timeInterval = getTimeInterval(7);
                            //如果在时间范围内
                            //查询表中是否存在该报警记录
                            List<DeviceAlarmRecord> alarmRecords = queryDeviceAlarmRecord(deviceAlarmRecord,
                                    timeInterval);
                            //如果没有有报警记录 新增记录
                            if (alarmRecords.size() == 0) {
                                //  新增
                                deviceAlarmRecordMapper.insert(deviceAlarmRecord);
                                //TODO 邮件推送暂时删除了
                            }
                        } else {
                            //不满足条件 同一信号监测到报警状态消除，对当前信号所有未处理历史报警打上已处理标签，并记录消除时间
                            LambdaUpdateWrapper<DeviceAlarmRecord> queryWrapper = new LambdaUpdateWrapper<>();
                            queryWrapper.eq(DeviceAlarmRecord::getSignalId, map.get("signalId").toString());
                            queryWrapper.eq(DeviceAlarmRecord::getAlarmClass, "1");
                            queryWrapper.eq(DeviceAlarmRecord::getAlarmLevel, deviceAlarmParameter.getAlarmLevel());
                            queryWrapper.in(DeviceAlarmRecord::getStatus, "01", "02")
                                    .set(DeviceAlarmRecord::getStatus, "09")
                                    .set(DeviceAlarmRecord::getFixTime, LocalDateTime.now());
                            //故障消除时间
                            deviceAlarmRecordMapper.update(null, queryWrapper);
                        }
                    }
                }
            }
        }

    }

    /**
     * 填充告警记录表
     *
     * @return 返回告警记录信息
     */
    public DeviceAlarmRecord FillingDeviceAlarmRecord(String type, DeviceAlarmParameter deviceAlarmParameter,
                                                      Map<String, Object> singalData, String value) {
        DeviceAlarmRecord deviceAlarmRecord = new DeviceAlarmRecord();
        MeterDevice device = meterDeviceMapper.selectById(singalData.get("meterDeviceId").toString());

        // 创建 SimpleDateFormat 对象，指定日期格式
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        if ("yx".equals(type)) {
            deviceAlarmRecord.setRecordId(IdUtil.fastSimpleUUID());
            deviceAlarmRecord.setStationId(singalData.get("stationId").toString());
            //变电站id
            deviceAlarmRecord.setSystemcode(singalData.get("systemcode").toString());
            //所属系统
            deviceAlarmRecord.setAlarmTime(DateUtil.toLocalDateTime(DateUtil.date()));
            //告警时间
            deviceAlarmRecord.setDeviceId(singalData.get("meterDeviceId").toString());
            //告警设备ID
            deviceAlarmRecord.setDeviceName(singalData.get("deviceName").toString());
            //告警设名称
            deviceAlarmRecord.setSignalId(singalData.get("signalId").toString());
            //告警信号id
            deviceAlarmRecord.setSignalName(singalData.get("signalName").toString());
            //告警信号名称
            deviceAlarmRecord.setAlarmValue(value);
            //告警值
            deviceAlarmRecord.setSignalUnit("");
            //值单位
            String alarmMessage = String.format("[%s]发生了%s！", singalData.get("deviceName").toString(),
                    singalData.get("signalName").toString());
            deviceAlarmRecord.setAlarmMessage(alarmMessage);
            //告警信息
            deviceAlarmRecord.setStatus("01");
            //状态
            deviceAlarmRecord.setAlarmLevel("03");
            deviceAlarmRecord.setAlarmClass("2");
            //2-设备报警
            deviceAlarmRecord.setRegion(device.getRegion());
            //区域标识
        }
        if ("yc".equals(type)) {
            deviceAlarmRecord.setRecordId(IdUtil.fastSimpleUUID());
            //变电站id
            deviceAlarmRecord.setStationId(singalData.get("stationId").toString());
            //所属系统
            deviceAlarmRecord.setSystemcode(singalData.get("systemcode").toString());
            //告警时间
            deviceAlarmRecord.setAlarmTime(DateUtil.toLocalDateTime(DateUtil.date()));
            //告警设备ID
            deviceAlarmRecord.setDeviceId(singalData.get("meterDeviceId").toString());
            //告警设备名称
            deviceAlarmRecord.setDeviceName(singalData.get("deviceName").toString());
            //告警信号id
            deviceAlarmRecord.setSignalId(singalData.get("signalId").toString());
            //告警信号名称
            deviceAlarmRecord.setSignalName(singalData.get("signalName").toString());
            //告警类型
            deviceAlarmRecord.setAlarmType(deviceAlarmParameter.getAlarmType());
            //告警等级
            deviceAlarmRecord.setAlarmLevel(deviceAlarmParameter.getAlarmLevel());
            //告警值
            deviceAlarmRecord.setAlarmValue(value);
            String sinalUnit = ObjUtil.isNotEmpty(singalData.get("signalUnit")) ?
                    singalData.get("signalUnit").toString() : "";
            //值单位
            deviceAlarmRecord.setSignalUnit(sinalUnit);
            //正常范围
            deviceAlarmRecord.setNormalRange(deviceAlarmParameter.getAlarmCondition());

            String alarmLevelname = getDictName(deviceAlarmParameter.getAlarmLevel(), "alarmLevel");
            String alarmMessage = String.format("信号[%s]监测值为[%s]%s,达到了设置的报警阈值范围[%s]，发生告警，告警级别为[%s]！",
                    singalData.get("signal_name").toString(), value, sinalUnit,
                    deviceAlarmParameter.getAlarmCondition(), alarmLevelname
            );
            //告警信息
            deviceAlarmRecord.setAlarmMessage(alarmMessage);
            //告警方式
            deviceAlarmRecord.setNoticeAction(deviceAlarmParameter.getNoticeType());
            //邮件通知
            deviceAlarmRecord.setNoticeEmails(deviceAlarmParameter.getNoticeEmails());
            //触发联动
            deviceAlarmRecord.setNoticeLinkages(deviceAlarmParameter.getNoticeLinkages());
            //状态
            deviceAlarmRecord.setStatus("01");
            //1-越限报警
            deviceAlarmRecord.setAlarmClass("1");
            //区域标识
            deviceAlarmRecord.setRegion(device.getRegion());

        }
        return deviceAlarmRecord;
    }

    //获取字典名称 根据项目编码和父项编码
    public String getDictName(String itemcode, String parentcode) {
        QueryWrapper<SysDictionaryItems> queryWrapperSysDictionaryItems = new QueryWrapper<>();
        String dictname = "";
        queryWrapperSysDictionaryItems.eq("itemcode", itemcode);
        queryWrapperSysDictionaryItems.eq("parentcode", parentcode);
        SysDictionaryItems sysDictionaryItems = sysDictionaryItemsMapper.selectOne(queryWrapperSysDictionaryItems);
        if (sysDictionaryItems != null) {
            dictname = sysDictionaryItems.getDictName();
        }
        return dictname;
    }

    /**
     * 根据告警规则 判断是否触发告警
     *
     * @return true 触发恭敬规则
     * false 没有触发告警规则
     */
    public boolean TriggerAlarm(DeviceAlarmParameter deviceAlarmParameter, String value) {
        List<String> listSymbol = new ArrayList<>();
        List<Double> values = new ArrayList<>();
        //      //获取告警触发条件
        String str = deviceAlarmParameter.getAlarmCondition();
        // 定义正则表达式
        String regex = "([0-9]+(\\.[0-9]+)?|value)([<>=!]{1,2})([0-9]+(\\.[0-9]+)?|value)";
        Pattern pattern = Pattern.compile(regex);
        Matcher matcher = pattern.matcher(str);

        // 查找并输出比较符号和数值
        while (matcher.find()) {
            String value1 = matcher.group(1);
            String operator = matcher.group(3);
            String value2 = matcher.group(4);

            if ("value".equals(value1)) {
                values.add(Double.parseDouble(value));
            } else {
                values.add(Double.valueOf(value1));
            }

            listSymbol.add(operator);

            if ("value".equals(value2)) {
                values.add(Double.parseDouble(value));
            } else {
                values.add(Double.valueOf(value2));
            }
        }

        // 比较操作
        boolean result = compare(values.get(0), listSymbol.get(0), values.get(1));
        boolean resultTwo = true;
        if (listSymbol.size() > 1) {
            resultTwo = compare(values.get(1), listSymbol.get(1), values.get(2));
        }
        return result && resultTwo;
    }

    private static boolean compare(Double value1, String operator, Double value2) {
        switch (operator) {
            case "<":
                return value1.compareTo(value2) < 0;
            case "<=":
                return value1.compareTo(value2) < 0 || value1.compareTo(value2) == 0;
            case ">":
                return value1.compareTo(value2) > 0;
            case ">=":
                return value1.compareTo(value2) > 0 || value1.compareTo(value2) == 0;
            case "==":
            case "=":
                return value1.compareTo(value2) == 0;
            case "!=":
                return value1.compareTo(value2) != 0;
            default:
                throw new IllegalArgumentException("错误的告警规则: " + operator);
        }
    }

    //执行告警相关动作 插入数据库
    private void doAlarmAction(DeviceAlarmRecord alarmRecord) {
        //1.通过设备id 和信号id 查询告警参数设置表（告警规则）
        LambdaQueryWrapper<DeviceAlarmParameter> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(DeviceAlarmParameter::getDeviceId, alarmRecord.getDeviceId());
        queryWrapper.eq(DeviceAlarmParameter::getSignalId, alarmRecord.getSignalId());
        List<DeviceAlarmParameter> alarmParameters = deviceAlarmParameterMapper.selectList(queryWrapper);
        for (DeviceAlarmParameter alarmParameter : alarmParameters) {
            alarmRecord.setAlarmType(alarmParameter.getAlarmType());
            alarmRecord.setNoticeAction(alarmParameter.getNoticeType());
            alarmRecord.setNoticeLinkages(alarmParameter.getNoticeLinkages());
            //todo 发送邮件
            // pushEmail(alarmRecord,alarmParameter);
        }
        this.saveOrUpdate(alarmRecord);
    }

    /**
     * 判断当前时间在不在区间范围之内
     *
     * @return true 在区间范围之内
     * false 不在时间范围之内
     */
    public Map<String, Object> getTimeInterval(int clock) {
        int beforeclock = 0;
        Map<String, Object> map = new HashMap<>();
        // 获取当前时间
        LocalDateTime now = LocalDateTime.now();

        // 获取昨天的 7 点
        LocalDateTime yesterday7AM = LocalDateTime.of(now.minusDays(1).toLocalDate(), LocalTime.of(clock, 0));
        map.put("starttime", yesterday7AM);
        // 获取今天的 7 点
        LocalDateTime today7AM = LocalDateTime.of(now.toLocalDate(), LocalTime.of(clock, 0));
        LocalDateTime nextday7AM = LocalDateTime.of(now.plusDays(1).toLocalDate(), LocalTime.of(clock, 0));
        map.put("endtime", today7AM);
        // 判断时间是否在范围内
        if (now.isAfter(yesterday7AM) && now.isBefore(today7AM)) {
            beforeclock = 1;
        } else {
            beforeclock = 0;
            map.put("starttime", today7AM);
            map.put("endtime", nextday7AM);
        }
        map.put("beforeclock", beforeclock);
        return map;
    }

    /**
     * 根据条件查询告警记录表 判断是否存在告警记录
     *
     * @return true 存在告警记录
     * false 不存在告警记录
     */
    public List<DeviceAlarmRecord> queryDeviceAlarmRecord(DeviceAlarmRecord deviceAlarmRecord,
                                                          Map<String, Object> timeInterval) {
        LambdaQueryWrapper<DeviceAlarmRecord> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(DeviceAlarmRecord::getSignalId, deviceAlarmRecord.getSignalId());
        queryWrapper.eq(DeviceAlarmRecord::getAlarmLevel, deviceAlarmRecord.getAlarmLevel());
        LocalDateTime starttime = (LocalDateTime) timeInterval.get("starttime");
        LocalDateTime endtime = (LocalDateTime) timeInterval.get("endtime");
        queryWrapper.between(DeviceAlarmRecord::getAlarmTime, starttime, endtime);
        queryWrapper.in(DeviceAlarmRecord::getStatus, "01", "02");
        return this.list(queryWrapper);

    }
}
