package com.yfd.platform.modules.auxcontrol.service.impl;

import cn.hutool.core.convert.Convert;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.yfd.platform.modules.auxcontrol.domain.DeviceSignal;
import com.yfd.platform.modules.auxcontrol.mapper.DeviceSignalMapper;
import com.yfd.platform.modules.auxcontrol.service.IDeviceSignalService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.sql.Timestamp;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 变电站-辅控设备-信号 服务实现类
 * </p>
 *
 * @author zhengsl
 * @since 2025-04-23
 */
@Service
public class DeviceSignalServiceImpl extends ServiceImpl<DeviceSignalMapper, DeviceSignal> implements IDeviceSignalService {

    @Resource
    private DeviceSignalMapper deviceSignalMapper;

    /**********************************
     * 用途说明: 修改变电站辅控设备信号
     * 参数说明  deviceSignal 变电站辅控设备信号
     * 返回值说明: com.yfd.platform.config.ResponseResult 返回修改成功或者失败
     ***********************************/
    @Override
    public boolean updateDeviceSignalValue(String slaveIp, String address, String type, String value,
                                           String dateString) throws ParseException {

        DeviceSignal deviceSignal = deviceSignalMapper.selectOneDeviceSignal(slaveIp, type, address);
        if (deviceSignal != null && ObjectUtil.isNotEmpty(deviceSignal)) {
            try {
                if ("yx".equals(type)) {
                    // 遥信
                    deviceSignal.setYxValue(Integer.parseInt(value));
                } else {
                    // 遥测
                    deviceSignal.setYcValue(Convert.toBigDecimal(value));
                }
            } catch (NumberFormatException e) {
                // 处理转换异常，例如返回false或抛出更具体的异常
                return false;
            }

            deviceSignal.setLastmodifier("系统更新");
            // 设置最后修改时间
            Date parsedDate;
            if (StrUtil.isEmpty(dateString) || "null".equals(dateString)) {
                parsedDate = new Date();
            } else {
                SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss.SSS");
                parsedDate = dateFormat.parse(dateString);
            }
            Timestamp timestamp = new Timestamp(parsedDate.getTime());
            deviceSignal.setLastmodifydate(timestamp);

            // 更新并检查结果
            return this.updateById(deviceSignal);
        }

        return false;
    }
}
