package com.yfd.platform.modules.basedata.service.impl;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yfd.platform.modules.basedata.domain.WeatherLog;
import com.yfd.platform.modules.basedata.mapper.WeatherLogMapper;
import com.yfd.platform.modules.basedata.service.IWeatherLogService;
import com.yfd.platform.system.service.ISysDictionaryItemsService;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * <p>
 * 微气象数据日志表 服务实现类
 * </p>
 *
 *
 * @since 2023-04-27
 */
@Service
public class WeatherLogServiceImpl extends ServiceImpl<WeatherLogMapper, WeatherLog> implements IWeatherLogService {

    @Resource
    private ISysDictionaryItemsService sysDictionaryItemsService;

    /**********************************
     * 用途说明: 查询环境信息
     * 参数说明 stationId
     * 返回值说明: java.util.Map<java.lang.String,java.lang.Object>
     ***********************************/
    @Override
    public Map<String, Object> getWeatherLogList(String stationId) {

        LambdaQueryWrapper<WeatherLog> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(WeatherLog::getStationId, stationId).eq(WeatherLog::getDatastatus, "1").orderByDesc(WeatherLog::getTime);
        List<WeatherLog> list = this.list(queryWrapper);
        Map<String, List<WeatherLog>> collect =
                list.stream().filter(s -> StrUtil.isNotBlank(s.getType())).collect(Collectors.groupingBy(WeatherLog::getType));
        List<Map<String, Object>> environmentType = sysDictionaryItemsService.getDeviceByType("EnvironmentType");
        Map<String, Object> weatherDevice = new HashMap<>();
        for (Map<String, Object> map : environmentType) {
            String itemcode = map.get("itemcode").toString();
            String value = "";
            for (String type : collect.keySet()) {
                List<WeatherLog> weatherLogs = collect.get(type);
                if (type.equals(itemcode)) {
                    WeatherLog weatherLog = weatherLogs.get(0);
                    String value1 = weatherLog.getValue();
                    if (StrUtil.isNotBlank(value1)) {
                        value = value1;
                    }
                    break;
                }
            }
            String custom2 = map.get("custom2").toString();
            weatherDevice.put(custom2, value);
        }
        String environment = "气温" + weatherDevice.get("temperature") + "°C，气压" + weatherDevice.get("pressure") + "Kpa" +
                "，风速" + weatherDevice.get("windSpeed") + "m/s";
        weatherDevice.put("environment", environment);
        return weatherDevice;
    }
}
