package com.yfd.platform.modules.patroltask.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.date.DatePattern;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.util.*;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yfd.platform.component.ServerSendEventServer;
import com.yfd.platform.component.WebSocketServer;
import com.yfd.platform.component.mqtt.MqttGateway;
import com.yfd.platform.component.nettyclient.*;
import com.yfd.platform.config.AlarmTemplateConfig;
import com.yfd.platform.config.HttpServerConfig;
import com.yfd.platform.config.ParentConfig;
import com.yfd.platform.config.ResponseResult;
import com.yfd.platform.modules.basedata.domain.LinkageSignal;
import com.yfd.platform.modules.basedata.domain.Substation;
import com.yfd.platform.modules.basedata.domain.SubstationDevice;
import com.yfd.platform.modules.basedata.domain.SubstationPatroldevice;
import com.yfd.platform.modules.basedata.mapper.LinkageSignalMapper;
import com.yfd.platform.modules.basedata.mapper.SubstationDeviceMapper;
import com.yfd.platform.modules.basedata.mapper.SubstationMapper;
import com.yfd.platform.modules.basedata.mapper.SubstationPatroldeviceMapper;
import com.yfd.platform.modules.patroltask.domain.AlarmLog;
import com.yfd.platform.modules.patroltask.domain.TaskResult;
import com.yfd.platform.modules.patroltask.domain.TaskTodo;
import com.yfd.platform.modules.patroltask.mapper.AlarmLogMapper;
import com.yfd.platform.modules.patroltask.mapper.TaskResultMapper;
import com.yfd.platform.modules.patroltask.mapper.TaskTodoMapper;
import com.yfd.platform.modules.patroltask.service.IAlarmLogService;
import com.yfd.platform.modules.patroltask.service.ITaskTodoService;
import com.yfd.platform.modules.robotapi.service.IStationRobotService;
import com.yfd.platform.system.domain.Message;
import com.yfd.platform.system.domain.SysOrganization;
import com.yfd.platform.system.mapper.SysOrganizationMapper;
import com.yfd.platform.system.service.IMessageService;
import com.yfd.platform.system.service.ISysDictionaryItemsService;
import com.yfd.platform.utils.FileUtil;
import com.yfd.platform.utils.SecurityUtils;
import io.netty.util.CharsetUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import javax.imageio.ImageIO;
import java.awt.*;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.math.BigDecimal;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.sql.Timestamp;
import java.text.SimpleDateFormat;
import java.time.LocalDateTime;
import java.util.*;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * <p>
 * 告警日志表 服务实现类
 * </p>
 *
 * @since 2023-05-03
 */
@Service
@Slf4j
public class AlarmLogServiceImpl extends ServiceImpl<AlarmLogMapper, AlarmLog> implements IAlarmLogService {

    @Resource
    private SubstationDeviceMapper deviceMapper;

    @Resource
    private SubstationPatroldeviceMapper patroldeviceMapper;

    @Resource
    private TaskTodoMapper taskTodoMapper;
    @Resource
    private TaskResultMapper taskResultMapper;
    @Resource
    private SubstationMapper stationMapper;
    @Resource
    private SubstationDeviceMapper substationDeviceMapper;
    @Resource
    private HttpServerConfig Config;
    @Resource
    private AlarmLogMapper alarmLogMapper;
    @Resource
    private ITaskTodoService taskTodoService;
    @Resource
    private AlarmTemplateConfig alarmTemplateConfig;
    @Resource
    private SysOrganizationMapper sysOrganizationMapper;
    @Resource
    IStationRobotService stationNodeService;
    @Resource
    private HttpServerConfig httpServerConfig;
    @Resource
    private ParentConfig parentConfig;
    @Resource
    private IMessageService messageService;

    @Resource
    private LinkageSignalMapper linkageSignalMapper;

    @Resource
    private ISysDictionaryItemsService sysDictionaryItemsService;

    @Resource
    private MqttGateway mqttGateway;

    @Override
    public synchronized boolean updateTaskResult(JSONObject jsonObject) throws Exception {
        JSONArray resultList = jsonObject.getJSONArray("resultList");
        for (int i = 0; i < resultList.size(); i++) {
            JSONObject resultObject = resultList.getJSONObject(i);
            if (resultObject == null) {
                continue;
            }
            String resultId = resultObject.getStr("objectId");
            TaskResult taskResult = taskResultMapper.selectById(resultId);
            JSONArray results = resultObject.getJSONArray("results");
            //设置任务结果数据
            updateTaskResult(taskResult, results);

            taskResultMapper.updateById(taskResult);
            //更新当前任务状态及进度信息
            TaskTodo taskTodo = taskTodoMapper.selectById(taskResult.getTaskTodoId());
            int sumnum = taskTodo.getDeviceSumnum();
            // 赋值执行数量
            List<Map<String, Object>> list = taskTodoService.getTaskResultById(taskTodo.getTaskTodoId(), null, null, null);
            // 已送检点数
            long patrolCount = list.stream().filter(t -> !("0").equals(t.get("flag")) && !("1").equals(t.get("flag"))).count();
            // 失败的数量
            long failCount =
                    list.stream().filter(t -> ("4").equals(t.get("flag")) || ("6").equals(t.get("flag"))).count();
            // 异常数量
            long abnormalCount = list.stream().filter(t -> ("2").equals(t.get("valid1"))).count();
            taskTodo.setDeviceDeforeNum((int) patrolCount);
            taskTodo.setDeviceUnusualnum((int) abnormalCount);
            taskTodo.setDeviceFailureNum((int) failCount);
            int patrolnum = taskResultMapper.selectCount(
                    new LambdaQueryWrapper<TaskResult>().
                            eq(TaskResult::getTaskTodoId, taskResult.getTaskTodoId()).
                            ne(TaskResult::getFlag, "0")
            );
            int percent = 0;
            percent = patrolnum * 100 / sumnum;  //百分之多少
            if (!"100".equals(taskTodo.getTaskProgress())) {
                taskTodo.setTaskProgress(String.valueOf(percent));
            }
            boolean b = patrolnum == sumnum;
            if (b) {
                //全部完成
                taskTodo.setTaskState("1"); //已完成
                taskTodo.setEndTime(DateUtil.now());
            }
            taskTodo.setLastmodifydate(new Timestamp(DateUtil.current()));
            taskTodoMapper.updateById(taskTodo);
            // 向上级发送任务数据
            taskTodoService.sendTaskResult(taskTodo, taskResult);
            Map<String, Object> map = taskTodoService.getTaskTodoById(taskResult.getTaskTodoId(), null, null, null);
            String stationCode = taskResult.getStationCode();
//            if (StrUtil.isNotBlank(taskTodo.getControlNum())) {
//                if (b) {
//                    WebSocketServer.sendInfo("taskresult_" + stationCode, JSONUtil.toJsonStr(JSONUtil.parse(map)));
//                }
//            } else {
//                WebSocketServer.sendInfo("taskresult_" + stationCode, JSONUtil.toJsonStr(JSONUtil.parse(map)));
//            }
            WebSocketServer.sendInfo("taskresult_" + stationCode, JSONUtil.toJsonStr(JSONUtil.parse(map)));
            if (taskTodo.getTaskType().equals("4")) {
                String controlNum = taskTodo.getControlNum();
                List<LinkageSignal> linkageSignals =
                        linkageSignalMapper.selectList(new LambdaQueryWrapper<LinkageSignal>().eq(LinkageSignal::getControlNum, controlNum));
                if (linkageSignals.size() > 0) {
                    LinkageSignal linkageSignal = linkageSignals.get(0);
                    String linkageType = linkageSignal.getLinkageType();
                    if ("1".equals(linkageType)) {
                        //对于一键顺控任务调用顺控文件生成和处理方法
                        stationNodeService.createVideocfmresult(taskResult.getStationCode(), taskResult.getTaskCode(),
                                taskResult.getDesc());
                    }
                }

            }

        }
        return true;
    }

    @Override
    public boolean updateDeviceResult(JSONObject dataObject) throws Exception {
        JSONArray resultList = dataObject.getJSONArray("resultList");
        for (int i = 0; i < resultList.size(); i++) {
            JSONObject resultObject = resultList.getJSONObject(i);
            if (resultObject == null) {
                continue;
            }
            String objectId = resultObject.getStr("objectId");
            SubstationDevice substationDevice = substationDeviceMapper.selectById(objectId);
            JSONArray results = resultObject.getJSONArray("results");
            //设置点位结果数据

            Set<String> validList = new HashSet<>();
            List<String> valueList = new ArrayList<>();
            List<String> confList = new ArrayList<>();
            List<String> descList = new ArrayList<>();
            List<String> typeList = new ArrayList<>();
            String filename = "";
            JSONArray posArray = new JSONArray();
            JSONObject map = new JSONObject();
            String resImagePath = "";
            for (int a = 0; a < results.size(); a++) {
                JSONObject result = results.getJSONObject(a);
                String code = result.getStr("code");
                if ("2000".equals(code)) {
                    String type = result.getStr("type");
                    typeList.add(type);
                    String value = result.getStr("value");
                    String desc = result.getStr("desc");
                    String conf = result.getStr("conf");
                    if (ObjectUtil.isNotEmpty(conf)) {
                        confList.add(conf);
                    }
                    String valueType = result.getStr("type");
                    validList.add("1");
                    String posStr = "";
                    if (!"infrared".equals(result.getStr("type")) && !"sound".equals(result.getStr("type")) && !"yjsk"
                            .equals(result.getStr("type")) && !"isolator".equals(result.getStr("type"))) {
                        //图片 排除声音
                        //                    filename = StrUtil.subAfter(URLDecoder.decode(result.getStr
                        //                    ("resImagePath"), "utf-8"), "=",
                        //                            true);
                        filename = result.getStr("resImagePath");
                        String pos = result.getStr("pos");
                        // 将框坐标统一存储画框
                        if (StrUtil.isNotBlank(pos)) {
                            JSONArray jsonArray = JSONUtil.parseArray(pos);
                            // 写入内容到文件
                            if (jsonArray.size() > 0) {
                                JSONObject jsonObject = jsonArray.getJSONObject(0);
                                if (jsonObject.size() > 0) {
                                    posArray.addAll(jsonArray);
                                    JSONArray areas = jsonObject.getJSONArray("areas");
                                    JSONObject jsonObject1 = areas.getJSONObject(0);
                                    JSONObject jsonObject2 = areas.getJSONObject(1);
                                    posStr =
                                            " " + jsonObject1.getStr("x") + " " + jsonObject1.getStr("y") + " " + jsonObject2.getStr("x") + " " + jsonObject2.getStr("y");
                                }
                            }

                        }
                    } else {
                        resImagePath = result.getStr("resImagePath");
                    }
                    Map<String, Object> judgeresult = judgeAbnormal(result.getStr("type"), result.getStr("value"),
                            result.getStr("conf"), substationDevice.getDeviceId(), "");
                    // || "yjsk".equals(result.getStr("type"))
                    log.info("=======================发送结果=======================" + result.getStr("type"));
                    if (!("meter".equals(type) || "infrared".equals(type) || "sound".equals(type) || StrUtil.isBlank(type) || "yjsk".equals(result.getStr("type")) || "isolator".equals(result.getStr("type")))) {
                        // 图像缺陷分析类型
                        List<Map<String, Object>> pictureDefectAnalysisTypeList =
                                sysDictionaryItemsService.getDeviceByType(
                                        "PictureDefectAnalysisType");
                        Set<String> customList =
                                pictureDefectAnalysisTypeList.stream().filter(c -> ObjectUtil.isNotEmpty(c.get(
                                        "custom1"))).map(c -> c.get("custom1").toString()).collect(Collectors.toSet());
                        for (String custom : customList) {
                            if (JSONUtil.isTypeJSONArray(custom)) {
                                JSONArray jsonArray = JSONUtil.parseArray(custom);
                                for (int j = 0; j < jsonArray.size(); j++) {
                                    JSONObject jsonObject = jsonArray.getJSONObject(j);
                                    // 检查当前对象的key字段是否与目标key匹配
                                    if (jsonObject.getStr("key").equals(valueType)) {
                                        valueList.add(jsonObject.getStr("value"));
                                        break; // 找到后跳出循环
                                    }
                                }
                            }

                        }
                    } else {
                        log.info("==============================进入===========================" + result.getStr("type"));
                        valueList.add(value);
                        descList.add(desc);
                        desc = "";
                    }
                    log.info("=========================================================" + judgeresult.get(
                            "isNormal").toString());
                    //判定为异常
                    String resultStr = "";
                    if ("0".equals(judgeresult.get("isNormal").toString())) {
                        //判定为异常
                        validList.add("2");
                        resultStr = result.getStr("type") + " " + result.getStr("conf") + posStr + "\n";
                    }
                    if (StrUtil.isNotBlank(desc) && ("0".equals(judgeresult.get("isNormal").toString()) || valueType.contains("fhz") || valueType.contains("isolator"))) {
                        descList.add(desc);
                    }
                } else {
                    // 非2000的判定为失败
                    validList.add("0");
                    descList.add("识别失败");
                }
            }
            log.info("========================validList.toString()=================================" + validList.toString());
            // 识别成功
            String value = StrUtil.join(",", valueList);
            String descJoin = StrUtil.join(",", descList);
            String confJoin = StrUtil.join(",", confList);
            String now = DateUtil.now();
            String typeStr = StrUtil.join(",", typeList);
            String fileName = URLEncoder.encode(filename, "utf-8");
            String defectFilePath = fileName;
            if (StrUtil.isNotBlank(filename) && posArray.size() > 0) {
                String originfilename = Config.getSnapFilePath() + filename;
                String markedfilename = Config.getSnapFilePath() + StrUtil.replace(filename, ".jpg", "_识别.jpg");
                FileUtil.MarkImageRectangle(originfilename, posArray.toString(), markedfilename);
                //水印字体
                Font font = new Font("微软雅黑", Font.PLAIN, 40);
                //水印图片色彩
                Color color1 = new Color(31, 183, 55);
                FileUtil.addWaterMark(markedfilename, markedfilename, substationDevice.getDeviceName(), 20, -40, color1,
                        font);
                Color color2 = new Color(226, 7, 24);
                String alertinfo = descJoin + "   " + "置信度:" + confJoin + "   " + now;
                FileUtil.addWaterMark(markedfilename, markedfilename, alertinfo, 600, -40, color2, font);
                defectFilePath = URLEncoder.encode(StrUtil.replace(filename, ".jpg", "_识别.jpg"), "utf-8");//转码存储
            }
            map.putOpt("filePath", fileName);
            if (StrUtil.isBlank(defectFilePath)) {
                map.putOpt("defectFilePath", resImagePath);
            } else {
                map.putOpt("defectFilePath", defectFilePath);
            }
            map.putOpt("value", value);
            map.putOpt("desc", descJoin);
            map.putOpt("valueType", typeStr);
            map.putOpt("conf", confJoin);
            map.putOpt("time", now);
            map.putOpt("patroldeviceDate", now);
            String flag = "2";
            String valid = "1";
            // 判定巡视结果
            if (validList.contains("2")) {
                // 只要存在异常就判定为异常
                flag = "2";
                valid = "2";
            }
            if (validList.size() == 1 && validList.contains("1")) {
                // 都为正常的判定为正常
                flag = "2";
                valid = "1";
            }
            if (validList.size() == 1 && validList.contains("0")) {
                // 都为巡视失败的判定为失败
                flag = "4";
                valid = "0";
            }
            map.putOpt("flag", flag);
            map.putOpt("deviceId", substationDevice.getDeviceId());
            map.putOpt("recognitionType", substationDevice.getRecognitionTypeList());
            map.putOpt("valid", valid);
            map.putOpt("meterType", substationDevice.getMeterType());
            String analysisResult = "";
            if ("4".equals(flag)) {
                analysisResult = "识别失败";
            } else if ("2".equals(valid)) {
                analysisResult = "异常";
            } else {
                analysisResult = "正常";
            }
            map.putOpt("analysisResult", analysisResult);
            log.info("--------------------------" + map.toString());
            // 发送数据到前端
            WebSocketServer.sendInfo("deviceresult_" + substationDevice.getStationCode(), map.toString());
        }
        return true;
    }

    /**********************************
     * 用途说明: 设置任务结果数据
     * 参数说明 taskResult
     * 参数说明 result
     * 返回值说明: com.yfd.platform.modules.patroltask.domain.TaskResult
     ***********************************/
    private void updateTaskResult(TaskResult taskResult, JSONArray results) throws Exception {
        //正确 2000  图像数据错误 2001  算法分析失败 2002

        Set<String> validList = new HashSet<>();
        List<String> valueList = new ArrayList<>();
        List<String> confList = new ArrayList<>();
        List<String> descList = new ArrayList<>();
        List<String> typeList = new ArrayList<>();
        String filename = "";
        JSONArray posArray = new JSONArray();
        Map<String, Object> map = new HashMap<>();
//        String filePath1 = URLDecoder.decode(taskResult.getFilePath(), "utf-8");
//        String fileName = StrUtil.subAfter(filePath1, "/", true).replace("_原图.jpg", ".txt").replace("_原图.jpeg", ".txt").replace("_原图.JPG", ".txt").replace("_原图.png", ".txt");
        // 获取todo信息创建文件
//        TaskTodo taskTodo = taskTodoMapper.selectById(taskResult.getTaskTodoId());
//        String planStartTime = taskTodo.getPlanStartTime();
//        DateTime date = DateUtil.parse(planStartTime);
//        String dateFormat = DateUtil.format(date, "yyyyMMddHHmmss");
//        String recognition = taskTodo.getTaskName() + dateFormat;
//        String filePath = httpServerConfig.getSnapFilePath() + recognition + File.separator + fileName;
//        File file = cn.hutool.core.io.FileUtil.touch(filePath);
//        FileWriter writer = new FileWriter(file);
        for (int a = 0; a < results.size(); a++) {
            JSONObject result = results.getJSONObject(a);
            String code = result.getStr("code");
            if ("2000".equals(code)) {
                String type = result.getStr("type");
                typeList.add(type);
                taskResult.setValueType(result.getStr("type"));
                String value = result.getStr("value");
                String desc = result.getStr("desc");
                String conf = result.getStr("conf");
                if (ObjectUtil.isNotEmpty(conf)) {
                    confList.add(conf);
                }
                validList.add("1");
                String posStr = "";
                if (!"infrared".equals(result.getStr("type")) && !"sound".equals(result.getStr("type")) && !"yjsk".equals(result.getStr("type")) && !"isolator".equals(result.getStr("type"))) {
                    //图片 排除声音
//                    filename = StrUtil.subAfter(URLDecoder.decode(result.getStr("resImagePath"), "utf-8"), "=",
//                            true);
                    filename = result.getStr("resImagePath");
                    String pos = result.getStr("pos");
                    // 将框坐标统一存储画框
                    if (StrUtil.isNotBlank(pos)) {
                        JSONArray jsonArray = JSONUtil.parseArray(pos);
                        // 写入内容到文件
                        if (jsonArray.size() > 0) {
                            JSONObject jsonObject = jsonArray.getJSONObject(0);
                            if (jsonObject.size() > 0) {
                                posArray.addAll(jsonArray);
                                JSONArray areas = jsonObject.getJSONArray("areas");
                                JSONObject jsonObject1 = areas.getJSONObject(0);
                                JSONObject jsonObject2 = areas.getJSONObject(1);
                                posStr = " " + jsonObject1.getStr("x") + " " + jsonObject1.getStr("y") + " " + jsonObject2.getStr("x") + " " + jsonObject2.getStr("y");
                            }
                        }

                    }
                } else {
                    taskResult.setDefectFilePath(taskResult.getFilePath());
                }
                String valueType = taskResult.getValueType();
                Map<String, Object> judgeresult = judgeAbnormal(result.getStr("type"), result.getStr("value"),
                        result.getStr("conf"), taskResult.getDeviceId(), taskResult.getResultId());
                // || "yjsk".equals(result.getStr("type"))
                log.info("=======================发送结果=======================" + result.getStr("type"));
                if (!("meter".equals(type) || "infrared".equals(type) || "sound".equals(type) || StrUtil.isBlank(type) || "yjsk".equals(result.getStr("type")) || "isolator".equals(result.getStr("type")))) {
                    // 图像缺陷分析类型
                    List<Map<String, Object>> pictureDefectAnalysisTypeList = sysDictionaryItemsService.getDeviceByType(
                            "PictureDefectAnalysisType");
                    Set<String> customList =
                            pictureDefectAnalysisTypeList.stream().filter(c -> ObjectUtil.isNotEmpty(c.get(
                                    "custom1"))).map(c -> c.get("custom1").toString()).collect(Collectors.toSet());
                    for (String custom : customList) {
                        if (JSONUtil.isTypeJSONArray(custom)) {
                            JSONArray jsonArray = JSONUtil.parseArray(custom);
                            for (int i = 0; i < jsonArray.size(); i++) {
                                JSONObject jsonObject = jsonArray.getJSONObject(i);
                                // 检查当前对象的key字段是否与目标key匹配
                                if (jsonObject.getStr("key").equals(valueType)) {
                                    valueList.add(jsonObject.getStr("value"));
                                    break; // 找到后跳出循环
                                }
                            }
                        }

                    }
                } else {
                    log.info("==============================进入===========================" + result.getStr(
                            "type"));
                    valueList.add(value);
                    descList.add(desc);
                    desc = "";
                }
                log.info("=========================================================" + judgeresult.get(
                        "isNormal").toString());
                //判定为异常
//                String resultStr = "";
                if ("0".equals(judgeresult.get("isNormal").toString())) {
                    map.put("alarmlevel", judgeresult.get("alarmlevel"));
                    //判定为异常
                    validList.add("2");
//                    resultStr = result.getStr("type") + " " + result.getStr("conf") + posStr + "\n";
                }

//                writer.write(resultStr);
                if (StrUtil.isNotBlank(desc) && ("0".equals(judgeresult.get("isNormal").toString()) || valueType.contains("fhz") || valueType.contains("isolator"))) {
                    descList.add(desc);
                }
            } else {
                // 非2000的判定为失败
                validList.add("0");
                descList.add("识别失败");
            }
        }
        log.info("========================validList.toString()=================================" + validList.toString());
//        writer.close();
        // 识别成功
        taskResult.setValue(StrUtil.join(",", valueList));
        String descJoin = StrUtil.join(",", descList);
        String confJoin = StrUtil.join(",", confList);
        String now = DateUtil.now();
        if (StrUtil.isNotBlank(filename) && StrUtil.isNotBlank(taskResult.getFilePath()) && posArray.size() > 0) {
            String originfilename = Config.getSnapFilePath() + filename;
            String markedfilename = Config.getSnapFilePath() + StrUtil.replace(filename, ".jpg", "_识别.jpg");
            FileUtil.MarkImageRectangle(originfilename, posArray.toString(), markedfilename);
            //水印字体
            Font font = new Font("微软雅黑", Font.PLAIN, 40);
            //水印图片色彩
            Color color1 = new Color(31, 183, 55);
            FileUtil.addWaterMark(markedfilename, markedfilename, taskResult.getDeviceName(), 20, -40, color1, font);
            Color color2 = new Color(226, 7, 24);
            String alertinfo = descJoin + "   " + "置信度:" + confJoin + "   " + now;
            FileUtil.addWaterMark(markedfilename, markedfilename, alertinfo, 600, -40, color2, font);
            String defectFilePath =
                    URLEncoder.encode(StrUtil.replace(filename, ".jpg", "_识别.jpg"), "utf-8");//转码存储
            taskResult.setDefectFilePath(defectFilePath);
            taskResult.setRectangle(posArray.toString());
        }
        //识别结果描述  示例：表计表盘破损
        taskResult.setDesc(descJoin);
        //识别结果置信度  示例：0.99
        taskResult.setConf(confJoin);
        taskResult.setTime(now);
        taskResult.setLastmodifydate(new Timestamp(DateUtil.current()));
        taskResult.setValueType(StrUtil.join(",", typeList));
        // 判定巡视结果
        if (validList.contains("2")) {
            // 只要存在异常就判定为异常
            taskResult.setFlag("2");
            taskResult.setValid("2");
            // 根据巡视点位判断是否生成报警
            if ("1".equals(taskResult.getIsAlarm())) {
                log.info("=========================生成报警记录====================");
                //生成报警记录
                createTaskAlarmLog(taskResult, map);
            }
        }
        if (validList.size() == 1 && validList.contains("1")) {
            // 都为正常的判定为正常
            taskResult.setFlag("2");
            taskResult.setValid("1");
        }
        if (validList.size() == 1 && validList.contains("0")) {
            // 都为巡视失败的判定为失败
            taskResult.setFlag("4");
            taskResult.setValid("0");
        }
        //TODO 新增加三相对比
//        sendNonCoherentMsg1(taskResult, 0);
    }

    /**********************************
     * 用途说明: 发送非同相告警
     * 参数说明 taskResult
     * 返回值说明: void
     ***********************************/
    @Override
    public void sendNonCoherentMsg1(TaskResult taskResult, double alertValue) throws Exception {
        log.info("=========================三相对比==============================" + taskResult.toString());
        SubstationDevice substationDevice1 = deviceMapper.selectById(taskResult.getDeviceId());
        if ("4".equals(substationDevice1.getRecognitionTypeList())) {
            substationDevice1.setMeterType("4");
        }
        String meterType = substationDevice1.getMeterType();
        if (!("4".equals(taskResult.getRecognitionType()) || "1".equals(taskResult.getRecognitionType()))) {
            return;
        }
        LambdaQueryWrapper<TaskResult> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.ne(TaskResult::getResultId, taskResult.getResultId()).eq(TaskResult::getTaskTodoId,
                taskResult.getTaskTodoId()).eq(TaskResult::getAreaId,
                taskResult.getAreaId()).eq(TaskResult::getBayId, taskResult.getBayId()).eq(TaskResult::getMainDeviceId, taskResult.getMainDeviceId()).eq(TaskResult::getComponentId, taskResult.getComponentId()).eq(TaskResult::getRecognitionType, taskResult.getRecognitionType()).eq(TaskResult::getFlag, "2");
        List<Map<String, Object>> mapList = taskResultMapper.selectMaps(queryWrapper);
        log.info("=========================三相对比数量==============================" + mapList.size());
        Map<String, Object> map1 = BeanUtil.beanToMap(taskResult);
        mapList.add(map1);
        if (mapList.size() < 3) {
            return;
        }
        log.info("=========================三相对比数量==============================" + mapList.size());
        mapList.forEach(r -> {
            String deviceId = r.get("deviceId").toString();
            SubstationDevice substationDevice = deviceMapper.selectById(deviceId);
            String meterType1 = substationDevice.getMeterType();
            if ("4".equals(substationDevice.getRecognitionTypeList())) {
                meterType1 = "4";
            }
            r.put("meterType", meterType1);
            r.put("phase", substationDevice.getPhase());
            //            if (taskResult.getResultId().equals(r.get("resultId").toString())) {
            //                r.put("value", taskResult.getValue());
            //            }
        });
        log.info("=========================三相对比数据==============================" + mapList.toString());
        Map<String, List<Map<String, Object>>> meterTypeMap =
                mapList.stream().collect(Collectors.groupingBy(m -> m.get("meterType").toString()));
        List<Map<String, Object>> mapList1 = meterTypeMap.get(meterType);
        int phaseSize = mapList1.stream().collect(Collectors.groupingBy(m -> m.get("phase").toString())).size();
        if (!(mapList1.size() == 3 && phaseSize == 3)) {
            return;
        }
        Map<String, String> map = new HashMap<>();
        // 三相对比
        if ("1".equals(taskResult.getRecognitionType())) {
            map.put("alarmtype", "9");
            map.put("alarmlevel", "4");
            alertValue = 1;
            // 仪表三相
            List<Double> values =
                    mapList1.stream().map(m -> NumberUtil.parseDouble(m.get("value").toString())).collect(Collectors.toList());
            List<Double> deviceNames =
                    mapList1.stream().map(m -> NumberUtil.parseDouble(m.get("deviceName").toString())).collect(Collectors.toList());
            Double max = CollectionUtil.max(values);
            Double min = CollectionUtil.min(values);
            String valueStr = StrUtil.join(",", values);
            String deviceNameStr = StrUtil.join(",", deviceNames);
            double difference = max - min;
            if (difference > alertValue) {
                String alarmInfo = String.format("仪表三相告警提示，点位【%s】，值【%s】，超过阈值%s告警！", deviceNameStr, valueStr,
                        alertValue);
                map.put("alarmInfo", alarmInfo);
                map.put("alertValue", alertValue + "");
                // 创建仪表三相告警
                createOtherAlarm(taskResult, map);
            }
        }

        if ("4".equals(taskResult.getRecognitionType())) {
            map.put("alarmtype", "3");
            map.put("alarmlevel", "4");
            alertValue = 10;
            // 仪表三相
            List<Double> values =
                    mapList1.stream().map(m -> NumberUtil.parseDouble(m.get("value").toString())).collect(Collectors.toList());
            List<String> deviceNames =
                    mapList1.stream().map(m -> m.get("deviceName").toString()).collect(Collectors.toList());
            Double max = CollectionUtil.max(values);
            Double min = CollectionUtil.min(values);
            String valueStr = StrUtil.join(",", values);
            String deviceNameStr = StrUtil.join(",", deviceNames);
            double difference = max - min;
            if (difference > alertValue) {
                String alarmInfo = String.format("三相温差报警告警提示，点位【%s】，值【%s】，超过阈值%s告警！", deviceNameStr, valueStr,
                        alertValue);
                map.put("alarmInfo", alarmInfo);
                map.put("alertValue", alertValue + "");
                // 创建三相温差告警
                createOtherAlarm(taskResult, map);
            }
        }
    }

    private boolean createOtherAlarm(TaskResult taskResult, Map<String, String> map) throws UnsupportedEncodingException {
        AlarmLog alarmLog = new AlarmLog();
        alarmLog.setId(IdUtil.fastSimpleUUID());
        String deviceid = taskResult.getDeviceId();
        SubstationDevice device = deviceMapper.selectById(deviceid);
        String alarmDate = DateUtil.now();
        Substation substation = stationMapper.selectOne(new LambdaQueryWrapper<Substation>().
                eq(Substation::getStationCode, device.getStationCode()));
        alarmLog.setStationCode(device.getStationCode());
        alarmLog.setStationId(substation.getStationId());
        alarmLog.setStationName(substation.getStationName());
        alarmLog.setAreaId(taskResult.getAreaId());
        alarmLog.setAreaName(taskResult.getAreaName());
        alarmLog.setBayId(taskResult.getBayId());
        alarmLog.setBayName(taskResult.getBayName());
        alarmLog.setMainDeviceId(taskResult.getMainDeviceId());
        alarmLog.setMainDeviceName(taskResult.getMainDeviceName());
        alarmLog.setComponentId(taskResult.getComponentId());
        alarmLog.setComponentName(taskResult.getComponentName());
        alarmLog.setDeviceId(taskResult.getDeviceId());
        alarmLog.setDeviceName(taskResult.getDeviceName());
        alarmLog.setPatroldeviceCode(taskResult.getPatroldeviceCode());
        alarmLog.setPatroldeviceName(taskResult.getPatroldeviceName());
        alarmLog.setMaterialId(taskResult.getMaterialId());
        TaskTodo taskTodo = taskTodoMapper.selectById(taskResult.getTaskTodoId());
        alarmLog.setTaskId(taskTodo.getTaskId());
        alarmLog.setTaskPatrolledId(taskResult.getTaskTodoId());
        alarmLog.setTaskResultId(taskResult.getResultId());
        alarmLog.setTaskCode(taskResult.getTaskCode());
        alarmLog.setTaskName(taskResult.getTaskName());
        alarmLog.setTaskAlarmType("1"); //'1:巡视任务告警；2:静默任务告警，3：巡视设备告警',
        //alarmLog.setMonitorType("");//非静默监视不填
        alarmLog.setAlarmType(ObjUtil.isNotEmpty(map.get("alarmtype")) ? map.get("alarmtype") : "");
        //'字典表<1>: = 超温报警<2>: = 温升报警<3>: = 三相温差报警<4>: = 三相对比报警<5>: = 声音异常<6>: = 外观异常<7>: = 仪表越限报警<8>: = 仪表超量程报警<9>: =
        // 仪表三相对比<10>: = 变位报警',
        alarmLog.setAlarmLevel(ObjUtil.isNotEmpty(map.get("alarmlevel")) ? map.get("alarmlevel") : "4");//
        // '1:预警；2:一般；3:严重；4;危急',  待判断细分
        alarmLog.setRecognitionType(taskResult.getRecognitionType());//'1:表计读取；2:位置状态识别；3:设备外观查看；4:红外测温；5:声音检测；6
        // :闪烁检测11:局放超声波检测12:局放地电压检测13:局放特高频检测101:环境温度检测102:环境湿度检测103:氧气浓度检测104SF6浓度检测',
        alarmLog.setFileType(taskResult.getFileType());//'1:红外图谱；2:可见光照片；3:音频4:视频5:识别图片，存在多个文件，文件类型用逗号分隔',
        //告警文件目录规范：地市名/区域名/变电站名/分类/年月
        String type = "缺陷";//缺陷、判别
        String alarmfilepath = String.format("%s/%s/%s/%s/%s/",
                substation.getCityName(),
                substation.getSectionName(),
                substation.getStationName(),
                type,
                DateUtil.format(DateUtil.date(), "yyyyMM")
        );
        log.info(
                "============================================拷贝原图到报警目录下============================================");
        //拷贝原图到报警目录下
        String fullfilename = URLDecoder.decode(taskResult.getFilePath(), "utf-8");
        String alarmfullfilename = alarmfilepath + StrUtil.subAfter(fullfilename, "/", true);
        FileUtil.copy(Config.getSnapFilePath() + fullfilename, Config.getAlarmFilePath() + alarmfullfilename, true);
        alarmLog.setFilePath(URLEncoder.encode(alarmfullfilename, "utf-8"));//原图文件名称（含路径）
        alarmLog.setDefectType(taskResult.getValueType());//'缺陷类别'
        log.info(
                "============================================拷贝识别图到报警目录下============================================");
        //拷贝识别图到报警目录下
        if (ObjUtil.isNotEmpty(taskResult.getDefectFilePath())) {
            String fulldefectfilename = URLDecoder.decode(taskResult.getDefectFilePath(), "utf-8");
            String alarmfilename = StrUtil.replace(StrUtil.subAfter(fulldefectfilename, "/", true), "识别", "缺陷告警");
            String fullalarmfile = alarmfilepath + alarmfilename;
            FileUtil.copy(Config.getSnapFilePath() + fulldefectfilename, Config.getAlarmFilePath() + fullalarmfile,
                    true);
            alarmLog.setDefectFilePath(URLEncoder.encode(fullalarmfile, "utf-8"));//标注了识别框和信息
        }
        log.info(
                "============================================图形识别结果（值）============================================");
        alarmLog.setValue(taskResult.getValue());//图形识别结果（值）
        alarmLog.setUnit(taskResult.getUnit());
        String alarminfo = map.get("alarmInfo");
        alarmLog.setContent(alarminfo);//告警描述信息
        alarmLog.setAlarmDate(alarmDate);//发送报警时间
        alarmLog.setCheckFlag("0");//'0:未核查；1:已审核；2.已修正'
        alarmLog.setRectangle(taskResult.getRectangle());//'格式：x1,y1,x2y2;x3,y3,x4,y4；多个报警框的左上角和右下角坐标'
        alarmLog.setDatastatus("1"); //初始创建
        alarmLog.setLastmodifier("系统创建");
        alarmLog.setLastmodifydate(new Timestamp(DateUtil.current()));
//        this.save(alarmLog);//保存此报警数据
        Map<String, Object> logMap = BeanUtil.beanToMap(alarmLog);
        String stationId = alarmLog.getStationId();
        LambdaQueryWrapper<AlarmLog> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AlarmLog::getDatastatus, "1").eq(AlarmLog::getStationId, stationId).eq(AlarmLog::getCheckFlag
                , 0);
        int count = this.count(queryWrapper);
        logMap.put("alarmCount", count);
        JSONObject jsonObject = new JSONObject(logMap);
        String stationCode = substation.getStationCode();
//        SysOrganization sysOrganization =
//                sysOrganizationMapper.selectOne(new LambdaUpdateWrapper<SysOrganization>().eq(SysOrganization::getOrgcode, stationCode));
//        String parentid = sysOrganization.getParentid();
        jsonObject.putOpt("deviceid", jsonObject.getStr("deviceId"));
        jsonObject.putOpt("channelId", taskResult.getPatroldeviceChannelcode());
        jsonObject.putOpt("deviceId", taskResult.getPatroldeviceCode());
        // 告警阈值
        jsonObject.putOpt("earlyMin", map.get("alertValue"));
        jsonObject.putOpt("earlyMax", map.get("alertValue"));
        jsonObject.putOpt("sameMin", map.get("alertValue"));
        jsonObject.putOpt("sameMax", map.get("alertValue"));
        jsonObject.putOpt("seriousMin", map.get("alertValue"));
        jsonObject.putOpt("seriousMax", map.get("alertValue"));
        jsonObject.putOpt("criticalMin", map.get("alertValue"));
        jsonObject.putOpt("criticalMax", map.get("alertValue"));
        // 如果是站端则区域也需要发送告警
//        if (!"0".equals(parentid)) {
//            WebSocketServer.sendInfo(parentid, jsonObject.toString());
//        }
        WebSocketServer.sendInfo(stationId, jsonObject.toString());
        return true;
    }

    /**********************************
     * 用途说明: 发送非同相告警
     * 参数说明 taskResult
     * 返回值说明: void
     ***********************************/
    private void sendNonCoherentMsg(TaskResult taskResult, double alertValue, String meterType) {
        String recognitionTypeList = taskResult.getRecognitionType();
        List<Map<String, Object>> nonCoherentAnalysis =
                taskResultMapper.getNonCoherentAnalysis(taskResult.getTaskTodoId(), null, null, null, null, null,
                        null, null);
        List<Map<String, Object>> mapList =
                nonCoherentAnalysis.stream().filter(m -> m.get("recognition_type").toString().equals(recognitionTypeList) && m.get("meterType").toString().equals(meterType)).collect(Collectors.toList());
        Set<String> phases = mapList.stream().map(m -> m.get("phase").toString()).collect(Collectors.toSet());
        Set<String> deviceIds = mapList.stream().map(m -> m.get("deviceId").toString()).collect(Collectors.toSet());
        // 如果包含相同相位或者相同点位则跳出
        if (phases.size() != mapList.size() || deviceIds.size() != mapList.size()) {
            return;
        }
        for (Map<String, Object> map : mapList) {
            if (ObjectUtil.isNotEmpty(map.get("value")) && StrUtil.isNotBlank(taskResult.getValue())) {
                BigDecimal value1 = new BigDecimal(taskResult.getValue());
                BigDecimal value2 = new BigDecimal(map.get("value").toString());
                double difference = Math.abs(value1.subtract(value2).doubleValue());
                if (difference <= alertValue) {
                    break;
                }
                Message message = new Message();
                TaskTodo taskTodo = taskTodoService.getById(taskResult.getTaskTodoId());
                String taskName = taskResult.getTaskName();
                // 消息标题
                String title = taskName + "（" + taskTodo.getStartTime() + "）";
                message.setTitle(title);
                message.setContent("表记巡视非同相结果差值超阈，结果一：" + map.get("value").toString() + "结果二：" + taskResult.getValue() +
                        "单位：" + taskResult.getUnit() + "阈值：" + alertValue);
                message.setCreatetime(new Timestamp(System.currentTimeMillis()));
                // 告警触发
                message.setType("4");
                message.setStatus("1");
                // 发送告警
                messageService.save(message);
                return;

            }
        }
    }

    /**********************************
     * 用途说明: 发送非同源告警
     * 参数说明 taskResult
     * 返回值说明: void
     ***********************************/
    private void sendNonHomologousMsg(TaskResult taskResult, double alertValue) {
        String deviceId = taskResult.getDeviceId();
        String taskTodoId = taskResult.getTaskTodoId();
        LambdaQueryWrapper<TaskResult> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaskResult::getTaskTodoId, taskTodoId).eq(TaskResult::getDeviceId, deviceId).ne(TaskResult::getResultId, taskResult.getResultId());
        List<TaskResult> taskResultList = taskResultMapper.selectList(queryWrapper);
        String content = "";
        for (TaskResult taskresult : taskResultList) {
            // 表记
            if ("1".equals(taskResult.getRecognitionType()) && ObjectUtil.isNotEmpty(taskResult.getValue()) && ObjectUtil.isNotEmpty(taskresult.getValue())) {
                BigDecimal value1 = new BigDecimal(taskResult.getValue());
                BigDecimal value2 = new BigDecimal(taskresult.getValue());
                double difference = Math.abs(value1.subtract(value2).doubleValue());
                if (difference <= alertValue) {
                    break;
                }
                content =
                        "表记巡视非同源结果差值超阈，结果一：" + taskresult.getValue() + "结果二：" + taskResult.getValue() + "单位：" + taskResult.getUnit() + "阈值：" + alertValue;
            }
            // 测温
            if ("4".equals(taskResult.getRecognitionType()) && ObjectUtil.isNotEmpty(taskResult.getValue()) && ObjectUtil.isNotEmpty(taskresult.getValue())) {
                String value = taskResult.getValue();
                String value1 = taskresult.getValue();
                String[] split = value.split(",");
                String[] split1 = value1.split(",");
                if (split.length != 2 || split1.length != 2) {
                    break;
                }
//                BigDecimal v1 = new BigDecimal(split[0]);
                BigDecimal v2 = new BigDecimal(split[0]);
//                BigDecimal v3 = new BigDecimal(split1[0]);
                BigDecimal v4 = new BigDecimal(split1[0]);
                if (Math.abs(v2.subtract(v4).doubleValue()) <= alertValue) {
                    break;
                }
                content =
                        "红外测温非同源最大温度结果差值超阈，结果一：" + taskresult.getValue() + "结果二：" + taskResult.getValue() + "单位：" + taskResult.getUnit() + "阈值：" + alertValue;
            }

            if ("2".equals(taskResult.getRecognitionType())) {
                // 设备位置状态识别
                String desc1 = taskResult.getDesc();
                String desc2 = taskresult.getDesc();
                if (StrUtil.isNotBlank(desc1) && StrUtil.isNotBlank(desc2) && !desc1.equals(desc2)) {
                    // 报警
                    content = "设备位置状态识别非同源结果不一致，结果一：" + taskresult.getValue() + "结果二：" + taskResult.getValue() +
                            "单位：" + taskResult.getUnit();
                }
            }
            // 如果符合报警条件则报警
            if (StrUtil.isNotBlank(content)) {
                Message message = new Message();
                TaskTodo taskTodo = taskTodoService.getById(taskResult.getTaskTodoId());
                String taskName = taskResult.getTaskName();
                // 消息标题
                String title = taskName + "（" + taskTodo.getStartTime() + "）";
                message.setTitle(title);
                message.setContent(content);
                message.setCreatetime(new Timestamp(System.currentTimeMillis()));
                // 告警触发
                message.setType("4");
                message.setStatus("1");
                // 发送告警
                messageService.save(message);
            }
            break;
        }
    }

    /**********************************
     * 用途说明: 告警
     * 参数说明 taskResult 当前结果
     * 返回值说明: void
     ***********************************/
    private void sendMessage(TaskResult taskResult) {
        String deviceId = taskResult.getDeviceId();
        String taskTodoId = taskResult.getTaskTodoId();
        LambdaQueryWrapper<TaskResult> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TaskResult::getTaskTodoId, taskTodoId).eq(TaskResult::getDeviceId, deviceId).ne(TaskResult::getResultId, taskResult.getResultId());
        List<TaskResult> taskResultList = taskResultMapper.selectList(queryWrapper);
        SubstationDevice substationDevice = deviceMapper.selectById(taskResult.getDeviceId());
        String patroldeviceJson = substationDevice.getPatroldeviceJson();
        JSONArray jsonArray = JSONUtil.parseArray(patroldeviceJson);
        if (jsonArray.size() != 2) {
            return;
        }
        String alertvalue = "0.0";
        for (int i = 0; i < jsonArray.size(); i++) {
            JSONObject jsonObject = jsonArray.getJSONObject(i);  //取第二条
            if (ObjectUtil.isEmpty(jsonObject.get("alertvalue"))) {
                continue;
            } else {
                alertvalue = jsonObject.get("alertvalue").toString();
            }
        }

        double alertValue = NumberUtil.parseDouble(alertvalue);
        String content = "";
        for (TaskResult taskresult : taskResultList) {
            if ("1".equals(taskResult.getRecognitionType()) || "4".equals(taskResult.getRecognitionType())) {
                // 表记和测温
                if (ObjectUtil.isNotEmpty(taskResult.getValue()) && ObjectUtil.isNotEmpty(taskresult.getValue())) {
                    BigDecimal value1 = new BigDecimal(taskResult.getValue());
                    BigDecimal value2 = new BigDecimal(taskresult.getValue());
                    double difference = Math.abs(value1.subtract(value2).doubleValue());
                    if (difference <= alertValue) {
                        break;
                    }
                    if ("1".equals(taskResult.getRecognitionType())) {
                        content =
                                "表记巡视非同源结果差值超阈，结果一：" + taskresult.getValue() + "结果二：" + taskResult.getValue() +
                                        "单位：" + taskResult.getUnit() + "阈值：" + alertvalue;
                    } else {
                        content =
                                "红外测温非同源结果差值超阈，结果一：" + taskresult.getValue() + "结果二：" + taskResult.getValue() +
                                        "单位：" + taskResult.getUnit() + "阈值：" + alertvalue;
                    }

                }

            }
            if ("2".equals(taskResult.getRecognitionType())) {
                // 设备位置状态识别
                String desc1 = taskResult.getDesc();
                String desc2 = taskresult.getDesc();
                if (StrUtil.isNotBlank(desc1) && StrUtil.isNotBlank(desc2) && !desc1.equals(desc2)) {
                    // 报警
                    content = "设备位置状态识别非同源结果不一致，结果一：" + taskresult.getValue() + "结果二：" + taskResult.getValue() +
                            "单位：" + taskResult.getUnit();
                }
            }
            // 如果符合报警条件则报警
            if (StrUtil.isNotBlank(content)) {
                Message message = new Message();
                TaskTodo taskTodo = taskTodoService.getById(taskResult.getTaskTodoId());
                String taskName = taskResult.getTaskName();
                // 消息标题
                String title = taskName + "（" + taskTodo.getStartTime() + "）";
                message.setTitle(title);
                message.setContent(content);
                message.setCreatetime(new Timestamp(System.currentTimeMillis()));
                // 告警触发
                message.setType("4");
                message.setStatus("1");
                // 发送告警
                messageService.save(message);
                long count =
                        messageService.count(new LambdaQueryWrapper<Message>().eq(Message::getStatus, "1"));
                ServerSendEventServer.sendMessage(count + "");
            }
            break;
        }
        if (!"1".equals(taskResult.getRecognitionType()) || StrUtil.isNotBlank(content)) {
            return;
        }

        // 相位
        String recognitionTypeList = substationDevice.getRecognitionTypeList();
        String meterType = substationDevice.getMeterType();
        List<Map<String, Object>> nonCoherentAnalysis =
                taskResultMapper.getNonCoherentAnalysis(taskResult.getTaskTodoId(), null, null, null, null, null,
                        null, null);
        List<Map<String, Object>> mapList =
                nonCoherentAnalysis.stream().filter(m -> m.get("recognition_type").toString().equals(recognitionTypeList) && m.get("meterType").toString().equals(meterType)).collect(Collectors.toList());
        Set<String> phases = mapList.stream().map(m -> m.get("phase").toString()).collect(Collectors.toSet());
        Set<String> deviceIds = mapList.stream().map(m -> m.get("deviceId").toString()).collect(Collectors.toSet());
        // 如果包含相同相位或者相同点位则跳出
        if (phases.size() != mapList.size() || deviceIds.size() != mapList.size()) {
            return;
        }
        for (Map<String, Object> map : mapList) {
            if (ObjectUtil.isNotEmpty(map.get("value")) && StrUtil.isNotBlank(taskResult.getValue())) {
                BigDecimal value1 = new BigDecimal(taskResult.getValue());
                BigDecimal value2 = new BigDecimal(map.get("value").toString());
                double difference = Math.abs(value1.subtract(value2).doubleValue());
                if (difference <= alertValue) {
                    break;
                }
                Message message = new Message();
                TaskTodo taskTodo = taskTodoService.getById(taskResult.getTaskTodoId());
                String taskName = taskResult.getTaskName();
                // 消息标题
                String title = taskName + "（" + taskTodo.getStartTime() + "）";
                message.setTitle(title);
                message.setContent("表记巡视非同相结果差值超阈，结果一：" + map.get("value").toString() + "结果二：" + taskResult.getValue() +
                        "单位：" + taskResult.getUnit() + "阈值：" + alertvalue);
                message.setCreatetime(new Timestamp(System.currentTimeMillis()));
                // 告警触发
                message.setType("4");
                message.setStatus("1");
                // 发送告警
                messageService.save(message);
                return;

            }
        }

    }

    /**********************************
     * 用途说明: 静默监视查询
     * 参数说明 stationId
     * 参数说明 areaId
     * 参数说明 patrolDeviceName
     * 参数说明 monitorType
     * 返回值说明: java.util.List<java.util.Map<java.lang.String,java.lang.Object>>
     ***********************************/
    @Override
    public Page<Map<String, Object>> getAlarmListByType(Page<Map<String, Object>> page, String stationId,
                                                        String areaId, String patrolDeviceName, String monitorType) {
        return alarmLogMapper.getAlarmListByType(page, stationId, areaId, patrolDeviceName, monitorType);
    }

    /**********************************
     * 用途说明: 根据设备编号查询告警列表
     * 参数说明 page
     * 参数说明 patrolDeviceCode
     * 参数说明 monitorType
     * 参数说明 startDate
     * 参数说明 endDate
     * 返回值说明: com.baomidou.mybatisplus.extension.plugins.pagination.Page<java.util.Map<java.lang.String,java.lang
     * .Object>>
     ***********************************/
    @Override
    public Page<Map<String, Object>> getAlarmListPage(Page<Map<String, Object>> page, String patrolDeviceCode,
                                                      String monitorType, String startDate, String endDate) {
        return alarmLogMapper.getAlarmListPage(page, patrolDeviceCode, monitorType, startDate, endDate);
    }

    @Override
    public Page<Map<String, Object>> getAlarmLogPage(Page<Map<String, Object>> page, String stationCode,
                                                     String taskAlarmType, String alarmLevel, String checkFlag,
                                                     String startDate, String endDate) {

        LambdaQueryWrapper<AlarmLog> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AlarmLog::getStationCode, stationCode);
        if (StrUtil.isNotBlank(taskAlarmType)) {
            queryWrapper.eq(AlarmLog::getTaskAlarmType, taskAlarmType);
        }
        if (StrUtil.isNotBlank(alarmLevel)) {
            queryWrapper.eq(AlarmLog::getAlarmLevel, alarmLevel);
        }
        if (StrUtil.isNotBlank(checkFlag)) {
            queryWrapper.eq(AlarmLog::getCheckFlag, checkFlag);
        }
        if (StrUtil.isNotBlank(startDate) && StrUtil.isNotBlank(endDate)) {
            queryWrapper.le(AlarmLog::getAlarmDate, endDate).ge(AlarmLog::getAlarmDate, startDate);
        }
        queryWrapper.orderByDesc(AlarmLog::getAlarmDate);
        return this.pageMaps(page, queryWrapper);
    }

    @Override
    public boolean setAlarmLogStatus(AlarmLog alarmLog) {
        String currentUsername = SecurityUtils.getCurrentUsername();
        //String currentUsername = "admin";
        Timestamp timestamp = new Timestamp(System.currentTimeMillis());
        String format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(timestamp);
        alarmLog.setLastmodifier(currentUsername);
        alarmLog.setCheckUserName(currentUsername);
        alarmLog.setLastmodifydate(timestamp);
        alarmLog.setCheckDate(format);
        String custom1 = alarmLog.getCustom1();
        if (StrUtil.isNotBlank(custom1) && !"0".equals(custom1)) {
            int i = Integer.parseInt(custom1);
            LocalDateTime now = LocalDateTime.now();
            LocalDateTime localDateTime = now.plusMinutes(i);
            if ("1".equals(alarmLog.getTaskAlarmType())) {
                String deviceId = alarmLog.getDeviceId();
                deviceMapper.update(null, new LambdaUpdateWrapper<SubstationDevice>().eq(SubstationDevice::getDeviceId,
                        deviceId).set(SubstationDevice::getDelayedAlarmDate, localDateTime));
            } else if ("2".equals(alarmLog.getTaskAlarmType())) {
                AlarmLog alarmLog1 = this.getById(alarmLog.getId());
                SubstationPatroldevice substationPatroldevice =
                        patroldeviceMapper.selectOne(new LambdaUpdateWrapper<SubstationPatroldevice>().eq(SubstationPatroldevice::getPatroldeviceCode, alarmLog1.getPatroldeviceCode()));
                String monitorType = this.getSlienceType(alarmLog1.getMonitorType());
                String slienceParams = substationPatroldevice.getSlienceParams();
                JSONArray jsonArray = JSONUtil.parseArray(slienceParams);
                JSONArray slienceArray = new JSONArray();
                // 维护延时报警截止时间
                for (Object o : jsonArray) {
                    JSONObject jsonObject = JSONUtil.parseObj(o);
                    if (monitorType.equals(jsonObject.getStr("type"))) {
                        jsonObject.putOpt("delayed_alarm_date", LocalDateTimeUtil.formatNormal(localDateTime));
                    }
                    slienceArray.add(jsonObject);
                }
                LambdaUpdateWrapper<SubstationPatroldevice> updateWrapper = new LambdaUpdateWrapper<>();
                updateWrapper.eq(SubstationPatroldevice::getPatroldeviceId,
                        substationPatroldevice.getPatroldeviceId()).set(SubstationPatroldevice::getSlienceParams,
                        slienceArray.toString());
                patroldeviceMapper.update(null, updateWrapper);
            }
        }
        boolean ok = this.updateById(alarmLog);
        String id = alarmLog.getId();
        AlarmLog alarmLog1 = this.getById(id);
        Map<String, Object> logMap = BeanUtil.beanToMap(alarmLog1);
        String stationId = alarmLog.getStationId();
        LambdaQueryWrapper<AlarmLog> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AlarmLog::getDatastatus, "1").eq(AlarmLog::getStationId, stationId).eq(AlarmLog::getCheckFlag
                , 0);
        int count = this.count(queryWrapper);
        logMap.put("alarmCount", count);
        JSONObject jsonObject = new JSONObject(logMap);
        SysOrganization sysOrganization =
                sysOrganizationMapper.selectOne(new LambdaUpdateWrapper<SysOrganization>().eq(SysOrganization::getOrgcode, alarmLog1.getStationCode()));
        String parentid = sysOrganization.getParentid();
        if (!"0".equals(parentid)) {
            WebSocketServer.sendInfo("check_" + parentid, jsonObject.toString());
        }
        WebSocketServer.sendInfo("check_" + alarmLog1.getStationId(), jsonObject.toString());
        return ok;
    }

    @Override
    public Map<String, Object> getAlarmLogById(String id) {
        Map<String, Object> map = this.getMap(new LambdaQueryWrapper<AlarmLog>().eq(AlarmLog::getId, id));
        if (map == null) {
            return ResponseResult.successData(null);
        }
        String taskResultId = map.get("taskResultId").toString();
        TaskResult taskResult = taskResultMapper.selectById(taskResultId);
        if (taskResult != null) {
            map.put("deviceId", taskResult.getPatroldeviceCode());
            map.put("channelId", taskResult.getPatroldeviceChannelcode());
            map.put("patroldevicePos", taskResult.getPatroldevicePos());
        }
        return map;
    }

    private String getSlienceType(String type) {
        String sliencetype = "";
        if ("wcgz".equals(type) || "wcaqm".equals(type) || "ryjjph".equals(type) || "xy".equals(type) ||
                "person_enter".equals(type)) {
            sliencetype = "ryxw";
        } else {
            sliencetype = type;
        }
        return sliencetype;
    }

    //根据识别类型和结果值判断是否为异常
    private Map<String, Object> judgeAbnormal(String type, String value, String conf, String deviceid,
                                              String resultId) throws Exception {
        Map<String, Object> result = new HashMap<>();
        SubstationDevice substationDevice = deviceMapper.selectById(deviceid);
        String pictureAnalysisTypeList = substationDevice.getPictureAnalysisTypeList();
        JSONObject jsonObject = JSONUtil.parseObj(pictureAnalysisTypeList);
        String pictureDefectAnalysisType = jsonObject.getStr("PictureDefectAnalysisType");
        result.put("isNormal", "1");
        if (StrUtil.isNotBlank(pictureDefectAnalysisType)) {
            // 缺陷告警设置
            //            List<Map<String, Object>> mapList = sysDictionaryItemsService.getDeviceByType
            //            ("PictureDefectAnalysisType");
            if ("1".equals(value)) {
                result.put("isNormal", "0");
            }
            return result;
        }
        String PictureDiscriminateAnalysisType = jsonObject.getStr("PictureDiscriminateAnalysisType");
        if (StrUtil.isNotBlank(PictureDiscriminateAnalysisType)) {
            if ("1".equals(value)) {
                result.put("isNormal", "0");
            }
            return result;
        }
        if ("meter".equals(type)) {//仪表读数
            double resultvalue = Double.parseDouble(value);
            //String alarmlevel = "0";// '1:预警；2:一般；3:严重；4;危急',
            SubstationDevice device = deviceMapper.selectById(deviceid);
            if (ObjUtil.isEmpty(device.getEarlyMin()) || ObjUtil.isEmpty(device.getCriticalMax())) {
                //未设置上下限，判定为正常
                result.put("isNormal", "1");//判定为正常
            } else {
                if (NumberUtil.compare(resultvalue, device.getEarlyMin()) >= 0 && NumberUtil.compare(resultvalue,
                        device.getEarlyMax()) < 0) {
                    result.put("alarmlevel", "1");
                    result.put("isNormal", "0");
                } else if (NumberUtil.compare(resultvalue, device.getSameMin()) >= 0 && NumberUtil.compare(resultvalue,
                        device.getSameMax()) < 0) {
                    result.put("alarmlevel", "2");
                    result.put("isNormal", "0");
                } else if (NumberUtil.compare(resultvalue, device.getSeriousMin()) >= 0 && NumberUtil.compare(resultvalue,
                        device.getSeriousMax()) < 0) {
                    result.put("alarmlevel", "3");
                    result.put("isNormal", "0");
                } else if (NumberUtil.compare(resultvalue, device.getCriticalMin()) >= 0 && NumberUtil.compare(resultvalue,
                        device.getCriticalMax()) < 0) {
                    result.put("alarmlevel", "4");
                    result.put("isNormal", "0");
                } else {
                    result.put("isNormal", "1");//判定为正常
                }
            }

        } else if ("infrared".equals(type)) {
            //温度 最高温度，最低温度（以逗号分隔）
            String highTemp = value.split(",")[0];  //最高温度
            double resultvalue = Double.parseDouble(highTemp);
            //String alarmlevel = "0";// '1:预警；2:一般；3:严重；4;危急',
            SubstationDevice device = deviceMapper.selectById(deviceid);
            if (ObjUtil.isEmpty(device.getEarlyMin()) || ObjUtil.isEmpty(device.getCriticalMax())) {
                //未设置上下限，判定为正常
                result.put("isNormal", "1");//判定为正常
            } else {
                if (NumberUtil.compare(resultvalue, device.getEarlyMin()) >= 0 && NumberUtil.compare(resultvalue,
                        device.getEarlyMax()) < 0) {
                    result.put("alarmlevel", "1");
                    result.put("isNormal", "0");
                } else if (NumberUtil.compare(resultvalue, device.getSameMin()) >= 0 && NumberUtil.compare(resultvalue,
                        device.getSameMax()) < 0) {
                    result.put("alarmlevel", "2");
                    result.put("isNormal", "0");
                } else if (NumberUtil.compare(resultvalue, device.getSeriousMin()) >= 0 && NumberUtil.compare(resultvalue,
                        device.getSeriousMax()) < 0) {
                    result.put("alarmlevel", "3");
                    result.put("isNormal", "0");
                } else if (NumberUtil.compare(resultvalue, device.getCriticalMin()) >= 0 && NumberUtil.compare(resultvalue,
                        device.getCriticalMax()) < 0) {
                    result.put("alarmlevel", "4");
                    result.put("isNormal", "0");
                } else {
                    //判定为正常
                    result.put("isNormal", "1");
                }
            }

        } else if ("isolator".equals(type)) {
            //刀闸状态
            if ("1".equals(value)) {
                result.put("desc", "分");
                result.put("isNormal", "1");
            }
            if ("2".equals(value)) {
                result.put("desc", "合");
                result.put("isNormal", "1");
            }
            if ("3".equals(value)) {
                result.put("desc", "分位异常");
                result.put("isNormal", "0");
            }
            if ("4".equals(value)) {
                result.put("desc", "合位异常");
                result.put("isNormal", "0");
            }

        } else if ("yjsk".equals(type)) {
            //刀闸状态
            if ("2".equals(value) || "4".equals(value)) {
                result.put("isNormal", "0");
            } else {
                result.put("isNormal", "1");
            }
        } else if ("sound".equals(type)) {//声音  1 代表正常声音，2 代表异常声音
            double resultvalue = Double.parseDouble(value);
            //String alarmlevel = "0";// '1:预警；2:一般；3:严重；4;危急',
            SubstationDevice device = deviceMapper.selectById(deviceid);
            if (ObjUtil.isEmpty(device.getEarlyMin()) || ObjUtil.isEmpty(device.getCriticalMax())) {
                //未设置上下限，判定为正常
                result.put("isNormal", "1");//判定为正常
            } else {
                if (NumberUtil.compare(resultvalue, device.getEarlyMin()) >= 0 && NumberUtil.compare(resultvalue,
                        device.getCriticalMax()) < 0) { //异常
                    result.put("isNormal", "0");//判定为异常
                    result.put("alarmtype", "7");//<7>: = 声音超限制
                    if (NumberUtil.compare(resultvalue, device.getEarlyMin()) >= 0 && NumberUtil.compare(resultvalue,
                            device.getEarlyMax()) < 0) {
                        result.put("alarmlevel", "1");
                    } else if (NumberUtil.compare(resultvalue, device.getSameMin()) >= 0 && NumberUtil.compare(resultvalue,
                            device.getSameMax()) < 0) {
                        result.put("alarmlevel", "2");
                    } else if (NumberUtil.compare(resultvalue, device.getSeriousMin()) >= 0 && NumberUtil.compare(resultvalue,
                            device.getSeriousMax()) < 0) {
                        result.put("alarmlevel", "3");
                    } else if (NumberUtil.compare(resultvalue, device.getCriticalMin()) >= 0 && NumberUtil.compare(resultvalue,
                            device.getCriticalMax()) < 0) {
                        result.put("alarmlevel", "4");
                    } else {
                        result.put("alarmlevel", "4");
                    }
                } else if (NumberUtil.compare(resultvalue, device.getCriticalMax()) > 0) {
                    result.put("isNormal", "0");//判定为异常
                    result.put("alarmtype", "5");//<5>: = 声音异常
                } else {
                    result.put("isNormal", "1");//判定为正常
                }
            }

        } else if ("tx_pb".equals(type)) {
            //缺陷判别
            if ("1".equals(value)) {
                result.put("isNormal", "0");
            } else {
                result.put("isNormal", "1");
            }
        } else if ("light".equals(type)) {
            if ("red_on".equals(value)) {
                result.put("isNormal", "0");
            } else {
                result.put("isNormal", "1");
            }
        } else if ("switch".equals(type)) {
            //switch,light
            result.put("isNormal", "1");//正常
        } else if ("qrcode".equals(type)) {
            // 实物 ID
        } else {
            if ("1".equals(value)) {//外观缺陷
                if ("ws_ywzc".equals(type)
                        || "fhz_f".equals(type) || "fhz_h".equals(type)
                        || "kgg_ybf".equals(type) || "kgg_ybh".equals(type)
                        || "kk_f".equals(type) || "kk_h".equals(type)
                        || "zsd_l".equals(type) || "zsd_m".equals(type)
                        || "hxq_gjzc".equals(type) || "ywzt_yfzc".equals(type)
                        || "bjzc".equals(type) || "xmbhzc".equals(type)
                        || "gzzc".equals(type) || "aqmzc".equals(type)) {
                    result.put("isNormal", "1");//正常
                } else {
                    result.put("isNormal", "0");//异常
                    result.put("alarmtype", "6");//<6>: = 外观异常
                }

            } else {
                result.put("isNormal", "1");//正常
            }
        }
        return result;
    }

    /**********************************
     * 用途说明: 生成报警记录(点位)
     * 参数说明 taskResult
     * 参数说明 map
     * 返回值说明: boolean
     ***********************************/
    private boolean createTaskAlarmLog(TaskResult taskResult, Map<String, Object> map) throws UnsupportedEncodingException {
        log.info("===========================" + taskResult.toString());
        AlarmLog alarmLog = new AlarmLog();
        alarmLog.setId(IdUtil.fastSimpleUUID());
        String deviceid = taskResult.getDeviceId();
        SubstationDevice device = deviceMapper.selectById(deviceid);
        String alarmDate = DateUtil.now();
        Substation substation = stationMapper.selectOne(new LambdaQueryWrapper<Substation>().
                eq(Substation::getStationCode, device.getStationCode()));
        alarmLog.setStationCode(device.getStationCode());
        alarmLog.setStationId(substation.getStationId());
        alarmLog.setStationName(substation.getStationName());
        alarmLog.setAreaId(taskResult.getAreaId());
        alarmLog.setAreaName(taskResult.getAreaName());
        alarmLog.setBayId(taskResult.getBayId());
        alarmLog.setBayName(taskResult.getBayName());
        alarmLog.setMainDeviceId(taskResult.getMainDeviceId());
        alarmLog.setMainDeviceName(taskResult.getMainDeviceName());
        alarmLog.setComponentId(taskResult.getComponentId());
        alarmLog.setComponentName(taskResult.getComponentName());
        alarmLog.setDeviceId(taskResult.getDeviceId());
        alarmLog.setDeviceName(taskResult.getDeviceName());
        alarmLog.setPatroldeviceCode(taskResult.getPatroldeviceCode());
        alarmLog.setPatroldeviceName(taskResult.getPatroldeviceName());
        alarmLog.setMaterialId(taskResult.getMaterialId());
        TaskTodo taskTodo = taskTodoMapper.selectById(taskResult.getTaskTodoId());
        alarmLog.setTaskId(taskTodo.getTaskId());
        alarmLog.setTaskPatrolledId(taskResult.getTaskTodoId());
        alarmLog.setTaskResultId(taskResult.getResultId());
        alarmLog.setTaskCode(taskResult.getTaskCode());
        alarmLog.setTaskName(taskResult.getTaskName());
        alarmLog.setTaskAlarmType("1"); //'1:巡视任务告警；2:静默任务告警，3：巡视设备告警',
        //alarmLog.setMonitorType("");//非静默监视不填
        alarmLog.setAlarmType(ObjUtil.isNotEmpty(map.get("alarmtype")) ? map.get("alarmtype").toString() : "");//'字典表<1>: = 超温报警<2>: = 温升报警<3>: = 三相温差报警<4>: = 三相对比报警<5>: = 声音异常<6>: = 外观异常<7>: = 仪表越限报警<8>: = 仪表超量程报警<9>: = 仪表三相对比<10>: = 变位报警',
        alarmLog.setAlarmLevel(ObjUtil.isNotEmpty(map.get("alarmlevel")) ? map.get("alarmlevel").toString() : "4");// '1:预警；2:一般；3:严重；4;危急',  待判断细分
        alarmLog.setRecognitionType(taskResult.getRecognitionType());//'1:表计读取；2:位置状态识别；3:设备外观查看；4:红外测温；5:声音检测；6:闪烁检测11:局放超声波检测12:局放地电压检测13:局放特高频检测101:环境温度检测102:环境湿度检测103:氧气浓度检测104SF6浓度检测',
        alarmLog.setFileType(taskResult.getFileType());//'1:红外图谱；2:可见光照片；3:音频4:视频5:识别图片，存在多个文件，文件类型用逗号分隔',
        //告警文件目录规范：地市名/区域名/变电站名/分类/年月
        String type = "缺陷";//缺陷、判别
        String alarmfilepath = String.format("%s/%s/%s/%s/%s/",
                substation.getCityName(),
                substation.getSectionName(),
                substation.getStationName(),
                type,
                DateUtil.format(DateUtil.date(), "yyyyMM")
        );
        log.info(
                "============================================拷贝原图到报警目录下============================================");
        //拷贝原图到报警目录下
        String fullfilename = URLDecoder.decode(taskResult.getFilePath(), "utf-8");
        String alarmfullfilename = alarmfilepath + StrUtil.subAfter(fullfilename, "/", true);
        FileUtil.copy(Config.getSnapFilePath() + fullfilename, Config.getAlarmFilePath() + alarmfullfilename, true);
        alarmLog.setFilePath(URLEncoder.encode(alarmfullfilename, "utf-8"));//原图文件名称（含路径）
        alarmLog.setDefectType(taskResult.getValueType());//'缺陷类别'
        log.info(
                "============================================拷贝识别图到报警目录下============================================");
        //拷贝识别图到报警目录下
        if (ObjUtil.isNotEmpty(taskResult.getDefectFilePath())) {
            String fulldefectfilename = URLDecoder.decode(taskResult.getDefectFilePath(), "utf-8");
            String alarmfilename = StrUtil.replace(StrUtil.subAfter(fulldefectfilename, "/", true), "识别", "缺陷告警");
            String fullalarmfile = alarmfilepath + alarmfilename;
            FileUtil.copy(Config.getSnapFilePath() + fulldefectfilename, Config.getAlarmFilePath() + fullalarmfile,
                    true);
            alarmLog.setDefectFilePath(URLEncoder.encode(fullalarmfile, "utf-8"));//标注了识别框和信息
        }
        log.info(
                "============================================图形识别结果（值）============================================");
        alarmLog.setValue(taskResult.getValue());//图形识别结果（值）
        alarmLog.setUnit(taskResult.getUnit());
        String alarminfo = "";
        String value = taskResult.getValue();
        String unit = "";
        if (StrUtil.isNotBlank(taskResult.getUnit())) {
            unit = taskResult.getUnit();
        }
        if ("meter".equals(taskResult.getValueType())) {
            alarminfo = String.format("%s-%s的%s，识别结果为%s%s，判定为[%s]超限缺陷！", taskResult.getMainDeviceName(),
                    taskResult.getComponentName(), taskResult.getDeviceName(), value,
                    unit, taskResult.getDesc());
        }
        if ("infrared".equals(taskResult.getValueType())) {
            alarminfo = String.format("%s-%s的%s，识别温度：%s单位：%s，判定为[%s]超限缺陷！", taskResult.getMainDeviceName(),
                    taskResult.getComponentName(), taskResult.getDeviceName(), value,
                    unit, taskResult.getDesc());
        }
        if ("sound".equals(taskResult.getValueType())) {
            alarminfo = "声音检测异常！";
        }
        if (StrUtil.isBlank(alarminfo)) {
//            // 图像缺陷分析类型
//            List<Map<String, Object>> pictureDefectAnalysisTypeList = sysDictionaryItemsService.getDeviceByType(
//                    "PictureDefectAnalysisType");
//            String valueType = taskResult.getValueType();
//            Set<String> customList = pictureDefectAnalysisTypeList.stream().filter(c -> ObjectUtil.isNotEmpty(c.get(
//                    "custom1"))).map(c -> c.get("custom1").toString()).collect(Collectors.toSet());
//            for (String custom : customList) {
//                if (JSONUtil.isTypeJSONArray(custom)) {
//                    JSONArray jsonArray = JSONUtil.parseArray(custom);
//                    for (int i = 0; i < jsonArray.size(); i++) {
//                        JSONObject jsonObject = jsonArray.getJSONObject(i);
//                        // 检查当前对象的key字段是否与目标key匹配
//                        if (jsonObject.getStr("key").equals(valueType)) {
//                            // 如果匹配，取出对应的value字段的值
//                            value = jsonObject.getStr("value");
//                            taskResult.setValue(value);
//                            break; // 找到后跳出循环
//                        }
//                    }
//                }
//
//            }
            alarminfo = String.format("%s-%s的%s，识别结果为%s%s，判定为[%s]异常！", taskResult.getMainDeviceName(),
                    taskResult.getComponentName(), taskResult.getDeviceName(), value, unit
                    , taskResult.getDesc());
        }
        alarmLog.setContent(alarminfo);//告警描述信息
        alarmLog.setAlarmDate(alarmDate);//发送报警时间
        alarmLog.setCheckFlag("0");//'0:未核查；1:已审核；2.已修正'
        alarmLog.setRectangle(taskResult.getRectangle());//'格式：x1,y1,x2y2;x3,y3,x4,y4；多个报警框的左上角和右下角坐标'
        alarmLog.setDatastatus("1"); //初始创建
        alarmLog.setLastmodifier("系统创建");
        alarmLog.setLastmodifydate(new Timestamp(DateUtil.current()));
        this.save(alarmLog);//保存此报警数据
        LocalDateTime delayedAlarmDate = device.getDelayedAlarmDate();
        // 如果当前时间小于报警延迟时间则不报警
        if (delayedAlarmDate != null) {
            LocalDateTime parse = LocalDateTimeUtil.parse(alarmDate, "yyyy-MM-dd HH:mm:ss");
            if (!parse.isAfter(delayedAlarmDate)) {
                return true;
            }
        }
        Map<String, Object> logMap = BeanUtil.beanToMap(alarmLog);
        String stationId = alarmLog.getStationId();
        LambdaQueryWrapper<AlarmLog> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(AlarmLog::getDatastatus, "1").eq(AlarmLog::getStationId, stationId).eq(AlarmLog::getCheckFlag
                , 0);
        int count = this.count(queryWrapper);
        logMap.put("alarmCount", count);
        JSONObject jsonObject = new JSONObject(logMap);
        String stationCode = substation.getStationCode();
        SysOrganization sysOrganization =
                sysOrganizationMapper.selectOne(new LambdaUpdateWrapper<SysOrganization>().eq(SysOrganization::getOrgcode, stationCode));
        String parentid = sysOrganization.getParentid();
        jsonObject.putOpt("deviceid", jsonObject.getStr("deviceId"));
        //        if (ObjectUtil.isNotEmpty(map.get("patroldeviceCode")) && ObjectUtil.isNotEmpty(map.get(
        //                "patroldeviceChannelcode"))) {
        //            jsonObject.putOpt("channelId", map.get("patroldeviceChannelcode").toString());
        //            jsonObject.putOpt("deviceId", map.get("patroldeviceCode").toString());
        //        }
        jsonObject.putOpt("channelId", taskResult.getPatroldeviceChannelcode());
        jsonObject.putOpt("deviceId", taskResult.getPatroldeviceCode());
        // 告警阈值
        jsonObject.putOpt("earlyMin", device.getEarlyMin());
        jsonObject.putOpt("earlyMax", device.getEarlyMax());
        jsonObject.putOpt("sameMin", device.getSameMin());
        jsonObject.putOpt("sameMax", device.getSameMax());
        jsonObject.putOpt("seriousMin", device.getSeriousMin());
        jsonObject.putOpt("seriousMax", device.getSeriousMax());
        jsonObject.putOpt("criticalMin", device.getCriticalMin());
        jsonObject.putOpt("criticalMax", device.getCriticalMax());
        log.info(
                "============================================如果是站端则区域也需要发送告警============================================");
        // 如果是站端则区域也需要发送告警
        if (!"0".equals(parentid)) {
            WebSocketServer.sendInfo(parentid, jsonObject.toString());
        }
        WebSocketServer.sendInfo(stationId, jsonObject.toString());
        log.info(
                "============================================发送告警============================================");
        //        // 向算法管理平台发送告警信息
        //        JSONObject alarmContent = new JSONObject();
        //        alarmContent.putOnce("province_name", substation.getProvinceName());
        //        alarmContent.putOnce("city_name", substation.getCityName());
        //        alarmContent.putOnce("volt_level", substation.getVoltLevel());
        //        alarmContent.putOnce("station_name", substation.getStationName());
        //        alarmContent.putOnce("section_ip", substation.getStationIp());
        //        alarmContent.putOnce("node_id", substation.getNodeId());
        //        alarmContent.putOnce("msg_type", "alarm");
        //        JSONArray alarmArray = new JSONArray();
        //        JSONObject alarmObject = new JSONObject();
        //        alarmObject.putOnce("bay_name", "");
        //        alarmObject.putOnce("device_name", "");
        //        alarmObject.putOnce("point_name", "");
        //        alarmObject.putOnce("time", DateUtil.now());
        //
        //        // 获取坐标
        //        String rectangle = taskResult.getRectangle();
        //        JSONArray defectArray = new JSONArray();
        //        JSONObject defectObject = new JSONObject();
        //        defectObject.putOnce("type", alarmLog.getDefectType());
        //        defectObject.putOnce("confidence", taskResult.getConf());
        //        defectObject.putOnce("desc", alarmLog.getContent());
        //        if (StrUtil.isNotBlank(rectangle)) {
        //            JSONArray rectangleArray = JSONUtil.parseArray(rectangle);
        //            JSONArray differentArray = new JSONArray();
        //            for (int i = 0; i < rectangleArray.size(); i++) {
        //                JSONObject coorsObject = rectangleArray.getJSONObject(i);
        //                // 坐标数组
        //                JSONArray coorsArray = coorsObject.getJSONArray("areas");
        //                JSONObject different = new JSONObject();
        //                for (int j = 0; j < coorsArray.size(); j++) {
        //                    JSONObject dataObject = coorsArray.getJSONObject(j);
        //                    for (String data : dataObject.keySet()) {
        //                        different.putOnce(data + (j + 1), dataObject.getStr(data));
        //                        defectObject.putOpt(data + (j + 1), dataObject.getStr(data));
        //                    }
        //                }
        //                differentArray.add(different);
        //            }
        //            alarmObject.putOnce("different", differentArray);
        //        } else {
        //            alarmObject.putOnce("different", new JSONArray());
        //        }
        //        defectArray.add(defectObject);
        //        alarmObject.putOnce("defect", defectArray);
        //        alarmArray.add(alarmObject);
        //        alarmContent.putOnce("alarm", alarmArray);
        //
        //        // 原图
        //        String picRaw = "";
        //        if (ObjectUtil.isNotEmpty(taskResult.getFilePath())) {
        //            picRaw = URLDecoder.decode(taskResult.getFilePath(), "utf-8");
        //        }
        //        alarmObject.putOnce("pic_raw", picRaw);
        //        // 识别图
        //        String picDefect = "";
        //        if (ObjectUtil.isNotEmpty(taskResult.getDefectFilePath())) {
        //            picDefect = URLDecoder.decode(taskResult.getDefectFilePath(), "utf-8");
        //        }
        //        alarmObject.putOnce("pic_defect", picDefect);
        //
        //        //TODO 缺少基准图、判别图
        //        alarmObject.putOnce("pic_different", "");
        //        alarmObject.putOnce("pic_diff_base", "");
        //        //文件转化为图片
        //        Image srcImg = null;
        //        try {
        //            if (StrUtil.isNotBlank(picDefect)) {
        //                String imagePath = httpServerConfig.getSnapFilePath() + picDefect;
        //                String ext = imagePath.substring(imagePath.lastIndexOf(".") + 1);
        //                if ("jpg".equals(ext) || "JPG".equals(ext)) {
        //                    srcImg = ImageIO.read(new File(imagePath));
        //                    //获取图片的宽
        //                    int srcImgWidth = srcImg.getWidth(null);
        //                    //获取图片的高
        //                    int srcImgHeight = srcImg.getHeight(null);
        //                    alarmObject.putOnce("pic_width", srcImgWidth);
        //                    alarmObject.putOnce("pic_height", srcImgHeight);
        //                }
        //            } else {
        //                alarmObject.putOnce("pic_width", 0);
        //                alarmObject.putOnce("pic_height", 0);
        //            }
        //        } catch (IOException e) {
        //            log.error("发生错误", e);
        //            throw new RuntimeException(e.getMessage());
        //        }
        ////        mqttGateway.sendToMqtt( "nodes/java/user/alert/2", 2,alarmContent.toString());
        //
        //        // 区域巡视主机生成告警数据时，主动向上级系统上报告警并接收响应消息。
//        JSONObject alarmData = new JSONObject();
//        alarmData.set("patroldevice_name", alarmLog.getPatroldeviceName());
//        alarmData.set("patroldevice_code", alarmLog.getPatroldeviceCode());
//        alarmData.set("task_name", alarmLog.getTaskName());
//        alarmData.set("task_code", alarmLog.getTaskCode());
//        alarmData.set("device_name", alarmLog.getDeviceName());
//        alarmData.set("device_id", alarmLog.getDeviceId());
//        alarmData.set("alarm_level", alarmLog.getAlarmLevel());
//        alarmData.set("alarm_type", alarmLog.getAlarmType());
//        alarmData.set("recognition_type", alarmLog.getRecognitionType());
//        alarmData.set("defect_type", alarmLog.getDefectType());
//        alarmData.set("file_type", alarmLog.getFileType());
//        alarmData.set("file_path", alarmLog.getFilePath());
//        alarmData.set("value", alarmLog.getPatroldeviceName());
//        alarmData.set("value_unit", alarmLog.getValue());
//        alarmData.set("unit", alarmLog.getUnit());
//        alarmData.set("time", taskResult.getTime());
//        alarmData.set("link_point", taskResult.getDeviceId());
//        alarmData.set("task_patrolled_id", alarmLog.getTaskPatrolledId());
//        alarmData.set("content", alarmLog.getContent());
//        this.sendTaskData("62", "", alarmLog.getStationCode(), alarmData.toString());
        return true;
    }

    public void sendTaskData(String Type, String Command, String Code, String Items) {
        String xml = MyXmlUtil.getXml(httpServerConfig.getPatrolServerid(), parentConfig.getParentId(), Type
                , Command, Code, Items);
        MyMessageProtocol messageProtocol = new MyMessageProtocol();
        messageProtocol.setSenderSerialNo(BootSessionCache.getRequestSessionID());
        messageProtocol.setReceiverSerialNo(new Long(0L));
        messageProtocol.setSessionflag(0);
        messageProtocol.setXmllen(xml.getBytes(CharsetUtil.UTF_8).length);
        messageProtocol.setXmlcontent(xml.getBytes(CharsetUtil.UTF_8));
        if (BootNettyClientChannelCache.channelMapCache.size() > 0) {
            for (Map.Entry<String, BootNettyClientChannel> entry :
                    BootNettyClientChannelCache.channelMapCache.entrySet()) {
                BootNettyClientChannel bootNettyChannel = entry.getValue();
                if (bootNettyChannel != null && bootNettyChannel.getChannel().isOpen()) {
                    bootNettyChannel.getChannel().writeAndFlush(messageProtocol);
                } else {
                    BootNettyClientThread thread = new BootNettyClientThread(parentConfig.getParentIp(),
                            Integer.parseInt(parentConfig.getTcpPort()), parentConfig);
                    thread.start();
                }
            }
        } else {
            BootNettyClientThread thread = new BootNettyClientThread(parentConfig.getParentIp(),
                    Integer.parseInt(parentConfig.getTcpPort()), parentConfig);
            thread.start();
        }
    }

    /**********************************
     * 用途说明: 生成报警记录(静默)
     * 参数说明 jsonObject
     * 返回值说明: boolean
     ***********************************/
    @Override
    public boolean createAlarm(JSONObject jsonObject) throws Exception {
        JSONArray jsonArray = jsonObject.getJSONArray("resultList");
        if (jsonArray.size() > 0) {
            JSONObject jsonResult = jsonArray.getJSONObject(0);
            JSONArray results = jsonResult.getJSONArray("results");
            for (int i = 0; i < results.size(); i++) {
                JSONObject result = results.getJSONObject(i);
                if (result.getStr("type").equals("gzzc") || result.getStr("type").equals("aqmzc")) {
                    //工装和安全帽正常
                    continue;
                }
                //正确 2000  图像数据错误 2001  算法分析失败 2002
                String code = result.getStr("code");
                if ("2000".equals(code)) {
                    Thread.sleep(1500);//延迟1.5秒
                    AlarmLog alarmLog = new AlarmLog();
                    alarmLog.setId(IdUtil.fastSimpleUUID());
                    //摄像机ID
                    String deviceid = jsonObject.getStr("requestId");
                    //摄像机信息
                    SubstationPatroldevice patroldevice = patroldeviceMapper.selectById(deviceid);
                    String alarmDate = DateUtil.now();
                    String monitorType = this.getSlienceType(result.getStr("type"));
                    LocalDateTime delayedAlarmDate = null;
                    String slienceParams = patroldevice.getSlienceParams();
                    if (StrUtil.isNotEmpty(slienceParams)) {
                        JSONArray slienceArray = JSONUtil.parseArray(slienceParams);
                        //获取报警时间判断是否报警
                        for (Object o : slienceArray) {
                            JSONObject silentData = JSONUtil.parseObj(o);
                            if (monitorType.equals(silentData.getStr("type"))) {
                                String delayedAlarmDateStr = silentData.getStr("delayed_alarm_date");
                                // 将日期转换成yyyy-MM-dd HH:mm:ss格式
                                delayedAlarmDate = LocalDateTimeUtil.parse(delayedAlarmDateStr,
                                        DatePattern.NORM_DATETIME_PATTERN);
                            }
                        }
                    }

                    Substation substation = stationMapper.selectOne(new LambdaQueryWrapper<Substation>().
                            eq(Substation::getStationCode, patroldevice.getStationCode()));
                    alarmLog.setStationCode(patroldevice.getStationCode());
                    alarmLog.setStationId(substation.getStationId());
                    alarmLog.setStationName(substation.getStationName());
                    alarmLog.setAreaId(patroldevice.getAreaId());
                    alarmLog.setAreaName(patroldevice.getAreaName());
                    alarmLog.setBayId("");
                    alarmLog.setBayName("");
                    alarmLog.setMainDeviceId("");
                    alarmLog.setMainDeviceName("");
                    alarmLog.setComponentId("");
                    alarmLog.setComponentName(patroldevice.getCustom2()); //监视位置
                    alarmLog.setDeviceId("");
                    alarmLog.setDeviceName(patroldevice.getCustom2());//静默监视点位
                    alarmLog.setPatroldeviceCode(patroldevice.getPatroldeviceCode());
                    alarmLog.setPatroldeviceName(patroldevice.getPatroldeviceName());
                    alarmLog.setTaskName(result.getStr("desc") + "静默监视");
                    alarmLog.setTaskAlarmType("2"); //'1:巡视任务告警；2:静默任务告警，3：巡视设备告警',
                    alarmLog.setMonitorType(result.getStr("type"));//监视类型
                    alarmLog.setAlarmType("");//'字典表<1>: = 超温报警<2>: = 温升报警<3>: = 三相温差报警<4>: = 三相对比报警<5>: = 声音异常<6>: =
                    // 外观异常<7>: = 仪表越限报警<8>: = 仪表超量程报警<9>: = 仪表三相对比<10>: = 变位报警',
                    alarmLog.setAlarmLevel("2");// '1:预警；2:一般；3:严重；4;危急',  待判断细分
                    alarmLog.setRecognitionType("");//'1:表计读取；2:位置状态识别；3:设备外观查看；4:红外测温；5:声音检测；6:闪烁检测11:局放超声波检测12
                    // :局放地电压检测13:局放特高频检测101:环境温度检测102:环境湿度检测103:氧气浓度检测104SF6浓度检测',
                    alarmLog.setFileType(patroldevice.getVideoMode());//'1:红外图谱；2:可见光照片；3:音频4:视频5:识别图片，存在多个文件，文件类型用逗号分隔',
                    //告警文件目录规范：地市名/区域名/变电站名/分类/年月
                    String type = "缺陷";//缺陷、判别
                    String alarmfilepath = String.format("%s/%s/%s/%s/%s/",
                            substation.getCityName(),
                            substation.getSectionName(),
                            substation.getStationName(),
                            type,
                            DateUtil.format(DateUtil.date(), "yyyyMM")
                    );
                    //文件命名格式为：日期_时间_间隔名_设备名_点位名_类型.jpg
//                    String alarmfilename = String.format("%s_%s_%s_%s_%s_%s.jpg",
//                            DateUtil.format(DateUtil.date(), "yyyyMMdd"),
//                            DateUtil.format(DateUtil.date(), "HHmmss"),
//                            patroldevice.getAreaName(),
//                            patroldevice.getPatroldeviceName(),
//                            patroldevice.getCustom2(),
//                            "原图"
//                    );
                    String alarmfilename = String.format("%s_%s_%s.jpg",
                            patroldevice.getPatroldeviceId(),
                            patroldevice.getPatroldeviceCode(),
                            DateUtil.format(DateUtil.date(), "yyyyMMddHHmmss")
                    );
                    String resImageUrl = result.getStr("resImagePath");
                    String fullfilename = URLDecoder.decode(resImageUrl.split("=")[1], "utf-8");
                    String alarmfullfilename = alarmfilepath + alarmfilename;

                    FileUtil.copy(Config.getSnapFilePath() + fullfilename,
                            Config.getAlarmFilePath() + alarmfullfilename, true);
                    alarmLog.setFilePath(URLEncoder.encode(alarmfullfilename, "utf-8"));//原图文件名称（含路径）

                    String markedfilename = StrUtil.replace(alarmfullfilename, ".jpg", "判别.jpg");
                    String markedfullfilename = Config.getAlarmFilePath() + markedfilename;
                    FileUtil.MarkImageRectangle(Config.getAlarmFilePath() + alarmfullfilename, result.getStr("pos"),
                            markedfullfilename);
                    //水印字体
                    Font font = new Font("微软雅黑", Font.PLAIN, 40);
                    //水印图片色彩
                    Color color1 = new Color(31, 183, 55);
                    FileUtil.addWaterMark(markedfullfilename, markedfullfilename, patroldevice.getCustom2(), 20, -40,
                            color1, font);
                    Color color2 = new Color(226, 7, 24);
                    String alertinfo =
                            result.getStr("desc") + "   " + "置信度:" + result.getStr("conf") + "   " + result.getStr(
                                    "time");
                    FileUtil.addWaterMark(markedfullfilename, markedfullfilename, alertinfo, 600, -40, color2, font);

                    alarmLog.setDefectFilePath(URLEncoder.encode(markedfilename, "utf-8"));//标注了识别框和信息
                    alarmLog.setDefectType(result.getStr("type"));//'缺陷类别'
                    alarmLog.setValue(result.getStr("value"));//图形识别结果（值）
                    alarmLog.setAlarmDate(alarmDate);//发送报警时间
                    String alarminfo = getTemplateContent(result.getStr("type"), substation.getStationName(),
                            patroldevice.getAreaName(), patroldevice.getPatroldeviceName(), alarmLog.getAlarmDate(),
                            result.getStr("conf"), result.getStr("desc"));
                    alarmLog.setContent(alarminfo);//告警描述信息
                    alarmLog.setCheckFlag("0");//'0:未核查；1:已审核；2.已修正'
                    alarmLog.setRectangle(JSONUtil.toJsonStr(result.getJSONArray("pos")));//'格式：x1,y1,x2y2;x3,y3,x4,
                    // y4；多个报警框的左上角和右下角坐标'
                    alarmLog.setDatastatus("1"); //初始创建
                    alarmLog.setLastmodifier("系统创建");
                    alarmLog.setLastmodifydate(new Timestamp(DateUtil.current()));
                    this.save(alarmLog);//保存此报警数据
                    // 如果当前时间小于报警延迟时间则不报警(zhengsl)
                    if (delayedAlarmDate != null) {
                        LocalDateTime parse = LocalDateTimeUtil.parse(alarmDate, "yyyy-MM-dd HH:mm:ss");
                        if (!parse.isAfter(delayedAlarmDate)) {
                            return true;
                        }
                    }
                    Map<String, Object> logMap = BeanUtil.beanToMap(alarmLog);
                    String stationId = alarmLog.getStationId();
                    LambdaQueryWrapper<AlarmLog> queryWrapper = new LambdaQueryWrapper<>();
                    queryWrapper.eq(AlarmLog::getDatastatus, "1").eq(AlarmLog::getStationId, stationId).eq(AlarmLog::getCheckFlag, 0);
                    int count = this.count(queryWrapper);
                    logMap.put("alarmCount", count);
                    JSONObject jsonObject1 = new JSONObject(logMap);
                    String stationCode = substation.getStationCode();
                    SysOrganization sysOrganization =
                            sysOrganizationMapper.selectOne(new LambdaUpdateWrapper<SysOrganization>().eq(SysOrganization::getOrgcode, stationCode));
                    String parentid = sysOrganization.getParentid();
                    String channelinfo = patroldevice.getChannelinfo();
                    if (StrUtil.isNotBlank(channelinfo)) {
                        JSONArray jsonArray1 = JSONUtil.parseArray(channelinfo);
                        for (int j = 0; j < jsonArray1.size(); j++) {
                            JSONObject channelData = jsonArray1.getJSONObject(j);
                            // channel_type 1：可见光 2：红外
                            if ("1".equals(channelData.getStr("channel_type"))) {
                                String fromDevice = channelData.getStr("from_device");
                                // 1：摄像机
                                if ("1".equals(fromDevice)) {
                                    jsonObject1.putOpt("deviceId", patroldevice.getInternationalId());
                                    jsonObject1.putOpt("channelId", channelData.getStr("channel_code"));
                                } else {
                                    // 2：来源录像机
                                    jsonObject1.putOpt("deviceId", channelData.getStr("recorder_code"));
                                    jsonObject1.putOpt("channelId", channelData.getStr("recorder_channelcode"));
                                }
                                break;
                            }
                        }

                    }
                    if (!"0".equals(parentid)) {
                        WebSocketServer.sendInfo(parentid, jsonObject1.toString());
                    }
                    WebSocketServer.sendInfo(stationId, jsonObject1.toString());
                    // 区域巡视主机生成告警数据时，主动向上级系统上报告警并接收响应消息。 郑顺利注释 ，暂时不用向主机发送消息
//                    JSONObject alarmData = new JSONObject();
//                    alarmData.set("patroldevice_name",alarmLog.getPatroldeviceName());
//                    alarmData.set("patroldevice_code",alarmLog.getPatroldeviceCode());
//                    alarmData.set("alarm_level",alarmLog.getAlarmLevel());
//                    alarmData.set("monitor_type",alarmLog.getMonitorType());
//                    alarmData.set("file_type",alarmLog.getFileType());
//                    alarmData.set("file_path",alarmLog.getFilePath());
//                    alarmData.set("time",alarmLog.getAlarmDate());
//                    alarmData.set("content",alarmLog.getContent());
//                    this.sendTaskData("63","",alarmLog.getStationCode(),alarmData.toString());
                }
            }

        }

        return true;
    }

    /**********************************
     * 用途说明: 获取模板内容
     * 参数说明
     * 返回值说明: java.lang.String
     ***********************************/
    private String getTemplateContent(String type, String stationName, String areaName, String patrolDeviceName,
                                      String time, String value, String desc) {
        String templateContent = "";
        switch (type) {
            // 未戴安全帽
            case "wcaqm":
                templateContent = alarmTemplateConfig.getHelmetCamera();
                break;
            // 未穿工装
            case "wcgz":
                templateContent = alarmTemplateConfig.getClothCamera();
                break;
            // 人员聚集、徘徊
            case "ryjjph":
                templateContent = alarmTemplateConfig.getIntensiveCamera();
                break;
            // 烟火
            case "hzyw":
                templateContent = alarmTemplateConfig.getFireCamera();
                break;
            // 渗漏油
            case "sly_dmyw":
                templateContent = alarmTemplateConfig.getLeakageCamera();
                break;
            // 人员进入
            case "person_enter":
                templateContent = alarmTemplateConfig.getpersonEnter();
                break;
            // 人员吸烟
            case "xy":
                templateContent = alarmTemplateConfig.getpersonSmoke();
                break;
        }
        if (StrUtil.isNotBlank(templateContent)) {
            return String.format(templateContent, stationName, areaName, patrolDeviceName, time, value, desc);
        } else {
            return "";
        }
    }

}
